@@ -30,6 +30,7 @@ import java.io.BufferedWriter;
 import java.io.EOFException;
 import java.io.File;
 import java.io.FileInputStream;
+import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStreamReader;
@@ -37,6 +38,7 @@ import java.io.OutputStreamWriter;
 import java.io.StringReader;
 import java.net.InetSocketAddress;
 import java.net.UnknownHostException;
+import java.util.Objects;
 import java.util.Set;
 import org.apache.bookkeeper.bookie.BookieException.InvalidCookieException;
 import org.apache.bookkeeper.bookie.BookieException.UnknownBookieIdException;
@@ -211,6 +213,12 @@ public class Cookie {
         return cBuilder;
     }
 
+    public static Cookie parseFromBytes(byte[] bytes) throws IOException {
+        try (BufferedReader reader = new BufferedReader(new StringReader(new String(bytes, UTF_8)))) {
+            return parse(reader).build();
+        }
+    }
+
     public void writeToDirectory(File directory) throws IOException {
         File versionFile = new File(directory,
             BookKeeperConstants.VERSION_FILENAME);
@@ -236,7 +244,7 @@ public class Cookie {
             throws BookieException {
         BookieId address = null;
         try {
-            address = Bookie.getBookieId(conf);
+            address = BookieImpl.getBookieId(conf);
         } catch (UnknownHostException e) {
             throw new UnknownBookieIdException(e);
         }
@@ -257,7 +265,7 @@ public class Cookie {
                                               Version version) throws BookieException {
         BookieId address = null;
         try {
-            address = Bookie.getBookieId(conf);
+            address = BookieImpl.getBookieId(conf);
         } catch (UnknownHostException e) {
             throw new UnknownBookieIdException(e);
         }
@@ -289,11 +297,11 @@ public class Cookie {
      * @return cookie builder object
      * @throws UnknownHostException
      */
-    static Builder generateCookie(ServerConfiguration conf)
+    public static Builder generateCookie(ServerConfiguration conf)
             throws UnknownHostException {
         Builder builder = Cookie.newBuilder();
         builder.setLayoutVersion(CURRENT_COOKIE_LAYOUT_VERSION);
-        builder.setBookieId(Bookie.getBookieId(conf).toString());
+        builder.setBookieId(BookieImpl.getBookieId(conf).toString());
         builder.setJournalDirs(Joiner.on(',').join(conf.getJournalDirNames()));
         builder.setLedgerDirs(encodeDirPaths(conf.getLedgerDirNames()));
         return builder;
@@ -310,7 +318,7 @@ public class Cookie {
     public static Versioned<Cookie> readFromRegistrationManager(RegistrationManager rm, ServerConfiguration conf)
             throws BookieException {
         try {
-            return readFromRegistrationManager(rm, Bookie.getBookieId(conf));
+            return readFromRegistrationManager(rm, BookieImpl.getBookieId(conf));
         } catch (UnknownHostException e) {
             throw new UnknownBookieIdException(e);
         }
@@ -346,7 +354,7 @@ public class Cookie {
      * @return cookie object
      * @throws IOException
      */
-    public static Cookie readFromDirectory(File directory) throws IOException {
+    public static Cookie readFromDirectory(File directory) throws IOException, FileNotFoundException {
         File versionFile = new File(directory, BookKeeperConstants.VERSION_FILENAME);
         try (BufferedReader reader = new BufferedReader(
                 new InputStreamReader(new FileInputStream(versionFile), UTF_8))) {
@@ -452,4 +460,23 @@ public class Cookie {
         return new Builder(oldCookie.layoutVersion, oldCookie.bookieId, oldCookie.journalDirs, oldCookie.ledgerDirs,
                 oldCookie.instanceId);
     }
+
+    @Override
+    public boolean equals(Object other) {
+        if (other instanceof Cookie) {
+            Cookie otherCookie = (Cookie) other;
+            return layoutVersion == otherCookie.layoutVersion
+                && Objects.equals(bookieId, otherCookie.bookieId)
+                && Objects.equals(journalDirs, otherCookie.journalDirs)
+                && Objects.equals(ledgerDirs, otherCookie.ledgerDirs)
+                && Objects.equals(instanceId, otherCookie.instanceId);
+        } else {
+            return false;
+        }
+    }
+
+    @Override
+    public int hashCode() {
+        return Objects.hash(bookieId, journalDirs, ledgerDirs, instanceId);
+    }
 }
