@@ -31,18 +31,18 @@ import static org.apache.bookkeeper.bookie.BookKeeperServerStats.STORAGE_GET_OFF
 import static org.apache.bookkeeper.bookie.BookKeeperServerStats.STORAGE_SCRUB_PAGES_SCANNED;
 import static org.apache.bookkeeper.bookie.BookKeeperServerStats.STORAGE_SCRUB_PAGE_RETRIES;
 
-import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.Lists;
 import com.google.common.util.concurrent.RateLimiter;
 
 import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufAllocator;
-import io.netty.util.ReferenceCountUtil;
+
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.EnumSet;
 import java.util.List;
 import java.util.Map;
 import java.util.NavigableMap;
@@ -87,13 +87,12 @@ import org.slf4j.LoggerFactory;
 )
 public class InterleavedLedgerStorage implements CompactableLedgerStorage, EntryLogListener {
     private static final Logger LOG = LoggerFactory.getLogger(InterleavedLedgerStorage.class);
-    public static final long INVALID_ENTRYID = -1;
 
     EntryLogger entryLogger;
     @Getter
     LedgerCache ledgerCache;
-    protected CheckpointSource checkpointSource;
-    protected Checkpointer checkpointer;
+    protected CheckpointSource checkpointSource = CheckpointSource.DEFAULT;
+    protected Checkpointer checkpointer = Checkpointer.NULL;
     private final CopyOnWriteArrayList<LedgerDeletionListener> ledgerDeletionListeners =
             Lists.newCopyOnWriteArrayList();
 
@@ -108,8 +107,6 @@ public class InterleavedLedgerStorage implements CompactableLedgerStorage, Entry
     // this indicates that a write has happened since the last flush
     private final AtomicBoolean somethingWritten = new AtomicBoolean(false);
 
-    private int pageSize;
-
     // Expose Stats
     @StatsDoc(
         name = STORAGE_GET_OFFSET,
@@ -127,9 +124,8 @@ public class InterleavedLedgerStorage implements CompactableLedgerStorage, Entry
     private OpStatsLogger pageScanStats;
     private Counter retryCounter;
 
-    @VisibleForTesting
     public InterleavedLedgerStorage() {
-        activeLedgers = new SnapshotMap<Long, Boolean>();
+        activeLedgers = new SnapshotMap<>();
     }
 
     @Override
@@ -137,9 +133,6 @@ public class InterleavedLedgerStorage implements CompactableLedgerStorage, Entry
                            LedgerManager ledgerManager,
                            LedgerDirsManager ledgerDirsManager,
                            LedgerDirsManager indexDirsManager,
-                           StateManager stateManager,
-                           CheckpointSource checkpointSource,
-                           Checkpointer checkpointer,
                            StatsLogger statsLogger,
                            ByteBufAllocator allocator)
             throws IOException {
@@ -148,9 +141,6 @@ public class InterleavedLedgerStorage implements CompactableLedgerStorage, Entry
             ledgerManager,
             ledgerDirsManager,
             indexDirsManager,
-            stateManager,
-            checkpointSource,
-            checkpointer,
             this,
             statsLogger,
             allocator);
@@ -160,9 +150,6 @@ public class InterleavedLedgerStorage implements CompactableLedgerStorage, Entry
                                         LedgerManager ledgerManager,
                                         LedgerDirsManager ledgerDirsManager,
                                         LedgerDirsManager indexDirsManager,
-                                        StateManager stateManager,
-                                        CheckpointSource checkpointSource,
-                                        Checkpointer checkpointer,
                                         EntryLogListener entryLogListener,
                                         StatsLogger statsLogger,
                                         ByteBufAllocator allocator) throws IOException {
@@ -171,34 +158,38 @@ public class InterleavedLedgerStorage implements CompactableLedgerStorage, Entry
                 ledgerManager,
                 ledgerDirsManager,
                 indexDirsManager,
-                stateManager,
-                checkpointSource,
-                checkpointer,
                 new EntryLogger(conf, ledgerDirsManager, entryLogListener, statsLogger.scope(ENTRYLOGGER_SCOPE),
                         allocator),
                 statsLogger);
     }
 
-    @VisibleForTesting
+    @Override
+    public void setStateManager(StateManager stateManager) {}
+
+    @Override
+    public void setCheckpointSource(CheckpointSource checkpointSource) {
+        this.checkpointSource = checkpointSource;
+    }
+
+    @Override
+    public void setCheckpointer(Checkpointer checkpointer) {
+        this.checkpointer = checkpointer;
+    }
+
     public void initializeWithEntryLogger(ServerConfiguration conf,
                 LedgerManager ledgerManager,
                 LedgerDirsManager ledgerDirsManager,
                 LedgerDirsManager indexDirsManager,
-                StateManager stateManager,
-                CheckpointSource checkpointSource,
-                Checkpointer checkpointer,
                 EntryLogger entryLogger,
                 StatsLogger statsLogger) throws IOException {
         checkNotNull(checkpointSource, "invalid null checkpoint source");
         checkNotNull(checkpointer, "invalid null checkpointer");
         this.entryLogger = entryLogger;
         this.entryLogger.addListener(this);
-        this.checkpointSource = checkpointSource;
-        this.checkpointer = checkpointer;
         ledgerCache = new LedgerCacheImpl(conf, activeLedgers,
                 null == indexDirsManager ? ledgerDirsManager : indexDirsManager, statsLogger);
-        gcThread = new GarbageCollectorThread(conf, ledgerManager, this, statsLogger.scope("gc"));
-        pageSize = conf.getPageSize();
+        gcThread = new GarbageCollectorThread(conf, ledgerManager, ledgerDirsManager,
+            this, statsLogger.scope("gc"));
         ledgerDirsManager.addLedgerDirsListener(getLedgerDirsListener());
         // Expose Stats
         getOffsetStats = statsLogger.getOpStatsLogger(STORAGE_GET_OFFSET);
@@ -270,6 +261,11 @@ public class InterleavedLedgerStorage implements CompactableLedgerStorage, Entry
         gcThread.enableForceGC();
     }
 
+    @Override
+    public void forceGC(boolean forceMajor, boolean forceMinor) {
+        gcThread.enableForceGC(forceMajor, forceMinor);
+    }
+
     @Override
     public boolean isInForceGC() {
         return gcThread.isInForceGC();
@@ -332,6 +328,13 @@ public class InterleavedLedgerStorage implements CompactableLedgerStorage, Entry
         return ledgerCache.ledgerExists(ledgerId);
     }
 
+    @Override
+    public boolean entryExists(long ledgerId, long entryId) throws IOException {
+        //Implementation should be as simple as what's below, but this needs testing
+        //return ledgerCache.getEntryOffset(ledgerId, entryId) > 0;
+        throw new UnsupportedOperationException("entry exists not supported");
+    }
+
     @Override
     public long getLastAddConfirmed(long ledgerId) throws IOException {
         Long lac = ledgerCache.getLastAddConfirmed(ledgerId);
@@ -345,7 +348,7 @@ public class InterleavedLedgerStorage implements CompactableLedgerStorage, Entry
                     lac = bb.readLong();
                     lac = ledgerCache.updateLastAddConfirmed(ledgerId, lac);
                 } finally {
-                    ReferenceCountUtil.release(bb);
+                    bb.release();
                 }
             }
         }
@@ -663,4 +666,39 @@ public class InterleavedLedgerStorage implements CompactableLedgerStorage, Entry
     public OfLong getListOfEntriesOfLedger(long ledgerId) throws IOException {
         return ledgerCache.getEntriesIterator(ledgerId);
     }
+
+    @Override
+    public void setLimboState(long ledgerId) throws IOException {
+        throw new UnsupportedOperationException(
+                "Limbo state only supported for DbLedgerStorage");
+    }
+
+    @Override
+    public boolean hasLimboState(long ledgerId) throws IOException {
+        throw new UnsupportedOperationException(
+                "Limbo state only supported for DbLedgerStorage");
+    }
+
+    @Override
+    public void clearLimboState(long ledgerId) throws IOException {
+        throw new UnsupportedOperationException(
+                "Limbo state only supported for DbLedgerStorage");
+    }
+
+    @Override
+    public EnumSet<StorageState> getStorageStateFlags() throws IOException {
+        return EnumSet.noneOf(StorageState.class);
+    }
+
+    @Override
+    public void setStorageStateFlag(StorageState flags) throws IOException {
+        throw new UnsupportedOperationException(
+                "Storage state only flags supported for DbLedgerStorage");
+    }
+
+    @Override
+    public void clearStorageStateFlag(StorageState flags) throws IOException {
+        throw new UnsupportedOperationException(
+                "Storage state flags only supported for DbLedgerStorage");
+    }
 }
