@@ -28,6 +28,7 @@ import io.netty.buffer.ByteBufAllocator;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.EnumSet;
 import java.util.List;
 import java.util.Optional;
 import java.util.PrimitiveIterator;
@@ -54,13 +55,14 @@ public interface LedgerStorage {
                     LedgerManager ledgerManager,
                     LedgerDirsManager ledgerDirsManager,
                     LedgerDirsManager indexDirsManager,
-                    StateManager stateManager,
-                    CheckpointSource checkpointSource,
-                    Checkpointer checkpointer,
                     StatsLogger statsLogger,
                     ByteBufAllocator allocator)
             throws IOException;
 
+    void setStateManager(StateManager stateManager);
+    void setCheckpointSource(CheckpointSource checkpointSource);
+    void setCheckpointer(Checkpointer checkpointer);
+
     /**
      * Start any background threads belonging to the storage system. For example, garbage collection.
      */
@@ -76,6 +78,11 @@ public interface LedgerStorage {
      */
     boolean ledgerExists(long ledgerId) throws IOException;
 
+    /**
+     * Whether an entry exists.
+     */
+    boolean entryExists(long ledgerId, long entryId) throws IOException, BookieException;
+
     /**
      * Fenced the ledger id in ledger storage.
      *
@@ -90,7 +97,28 @@ public interface LedgerStorage {
      * @param ledgerId Ledger ID.
      * @throws IOException
      */
-    boolean isFenced(long ledgerId) throws IOException;
+    boolean isFenced(long ledgerId) throws IOException, BookieException;
+
+    /**
+     * Set a ledger to limbo state.
+     * When a ledger is in limbo state, we cannot answer any requests about it.
+     * For example, if a client asks for an entry, we cannot say we don't have it because
+     * it may have been written to us in the past, but we are waiting for data integrity checks
+     * to copy it over.
+     */
+    void setLimboState(long ledgerId) throws IOException;
+
+    /**
+     * Check whether a ledger is in limbo state.
+     * @see #setLimboState(long)
+     */
+    boolean hasLimboState(long ledgerId) throws IOException;
+
+    /**
+     * Clear the limbo state of a ledger.
+     * @see #setLimboState(long)
+     */
+    void clearLimboState(long ledgerId) throws IOException;
 
     /**
      * Set the master key for a ledger.
@@ -115,7 +143,7 @@ public interface LedgerStorage {
     /**
      * Read an entry from storage.
      */
-    ByteBuf getEntry(long ledgerId, long entryId) throws IOException;
+    ByteBuf getEntry(long ledgerId, long entryId) throws IOException, BookieException;
 
     /**
      * Get last add confirmed.
@@ -124,7 +152,7 @@ public interface LedgerStorage {
      * @return last add confirmed.
      * @throws IOException
      */
-    long getLastAddConfirmed(long ledgerId) throws IOException;
+    long getLastAddConfirmed(long ledgerId) throws IOException, BookieException;
 
     /**
      * Wait for last add confirmed update.
@@ -188,7 +216,7 @@ public interface LedgerStorage {
 
     void setExplicitLac(long ledgerId, ByteBuf lac) throws IOException;
 
-    ByteBuf getExplicitLac(long ledgerId) throws IOException;
+    ByteBuf getExplicitLac(long ledgerId) throws IOException, BookieException;
 
     // for testability
     default LedgerStorage getUnderlyingLedgerStorage() {
@@ -202,6 +230,13 @@ public interface LedgerStorage {
         return;
     }
 
+    /**
+     * Force trigger Garbage Collection with forceMajor or forceMinor parameter.
+     */
+    default void forceGC(boolean forceMajor, boolean forceMinor) {
+        return;
+    }
+
     /**
      * Class for describing location of a generic inconsistency.  Implementations should
      * ensure that detail is populated with an exception which adequately describes the
@@ -275,4 +310,30 @@ public interface LedgerStorage {
      * @throws Exception
      */
     PrimitiveIterator.OfLong getListOfEntriesOfLedger(long ledgerId) throws IOException;
+
+    /**
+     * Get the storage state flags currently set for the storage instance.
+     */
+    EnumSet<StorageState> getStorageStateFlags() throws IOException;
+
+    /**
+     * Set a storage state flag for the storage instance.
+     * Implementations must ensure this method is atomic, and the flag
+     * is persisted to storage when the method returns.
+     */
+    void setStorageStateFlag(StorageState flags) throws IOException;
+
+    /**
+     * Clear a storage state flag for the storage instance.
+     * Implementations must ensure this method is atomic, and the flag
+     * is persisted to storage when the method returns.
+     */
+    void clearStorageStateFlag(StorageState flags) throws IOException;
+
+    /**
+     * StorageState flags.
+     */
+    enum StorageState {
+        NEEDS_INTEGRITY_CHECK
+    }
 }
