@@ -28,6 +28,7 @@ import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufAllocator;
 
 import java.io.IOException;
+import java.util.EnumSet;
 import java.util.List;
 import java.util.Optional;
 import java.util.PrimitiveIterator;
@@ -59,6 +60,8 @@ public class SortedLedgerStorage
     EntryMemTable memTable;
     private ScheduledExecutorService scheduler;
     private StateManager stateManager;
+    private ServerConfiguration conf;
+    private StatsLogger statsLogger;
     private final InterleavedLedgerStorage interleavedLedgerStorage;
 
     public SortedLedgerStorage() {
@@ -75,37 +78,47 @@ public class SortedLedgerStorage
                            LedgerManager ledgerManager,
                            LedgerDirsManager ledgerDirsManager,
                            LedgerDirsManager indexDirsManager,
-                           StateManager stateManager,
-                           CheckpointSource checkpointSource,
-                           Checkpointer checkpointer,
                            StatsLogger statsLogger,
                            ByteBufAllocator allocator)
             throws IOException {
+        this.conf = conf;
+        this.statsLogger = statsLogger;
 
         interleavedLedgerStorage.initializeWithEntryLogListener(
             conf,
             ledgerManager,
             ledgerDirsManager,
             indexDirsManager,
-            stateManager,
-            checkpointSource,
-            checkpointer,
             // uses sorted ledger storage's own entry log listener
             // since it manages entry log rotations and checkpoints.
             this,
             statsLogger,
             allocator);
 
+        this.scheduler = Executors.newSingleThreadScheduledExecutor(
+                new ThreadFactoryBuilder()
+                .setNameFormat("SortedLedgerStorage-%d")
+                .setPriority((Thread.NORM_PRIORITY + Thread.MAX_PRIORITY) / 2).build());
+    }
+
+    @Override
+    public void setStateManager(StateManager stateManager) {
+        interleavedLedgerStorage.setStateManager(stateManager);
+        this.stateManager = stateManager;
+    }
+    @Override
+    public void setCheckpointSource(CheckpointSource checkpointSource) {
+        interleavedLedgerStorage.setCheckpointSource(checkpointSource);
+
         if (conf.isEntryLogPerLedgerEnabled()) {
             this.memTable = new EntryMemTableWithParallelFlusher(conf, checkpointSource, statsLogger);
         } else {
             this.memTable = new EntryMemTable(conf, checkpointSource, statsLogger);
         }
-        this.scheduler = Executors.newSingleThreadScheduledExecutor(
-                new ThreadFactoryBuilder()
-                .setNameFormat("SortedLedgerStorage-%d")
-                .setPriority((Thread.NORM_PRIORITY + Thread.MAX_PRIORITY) / 2).build());
-        this.stateManager = stateManager;
+    }
+    @Override
+    public void setCheckpointer(Checkpointer checkpointer) {
+        interleavedLedgerStorage.setCheckpointer(checkpointer);
     }
 
     @VisibleForTesting
@@ -151,6 +164,12 @@ public class SortedLedgerStorage
         return true;
     }
 
+    @Override
+    public boolean entryExists(long ledgerId, long entryId) throws IOException {
+        // can probably be implemented as above, but I'm not going to test it
+        throw new UnsupportedOperationException("Not supported for SortedLedgerStorage");
+    }
+
     @Override
     public boolean setFenced(long ledgerId) throws IOException {
         return interleavedLedgerStorage.setFenced(ledgerId);
@@ -197,7 +216,7 @@ public class SortedLedgerStorage
     }
 
     @Override
-    public ByteBuf getEntry(long ledgerId, long entryId) throws IOException {
+    public ByteBuf getEntry(long ledgerId, long entryId) throws IOException, BookieException {
         if (entryId == BookieProtocol.LAST_ADD_CONFIRMED) {
             return getLastEntryId(ledgerId);
         }
@@ -351,6 +370,11 @@ public class SortedLedgerStorage
         interleavedLedgerStorage.forceGC();
     }
 
+    @Override
+    public void forceGC(boolean forceMajor, boolean forceMinor) {
+        interleavedLedgerStorage.forceGC(forceMajor, forceMinor);
+    }
+
     @Override
     public List<DetectedInconsistency> localConsistencyCheck(Optional<RateLimiter> rateLimiter) throws IOException {
         return interleavedLedgerStorage.localConsistencyCheck(rateLimiter);
@@ -372,4 +396,39 @@ public class SortedLedgerStorage
         PrimitiveIterator.OfLong entriesFromILSItr = interleavedLedgerStorage.getListOfEntriesOfLedger(ledgerId);
         return IteratorUtility.mergePrimitiveLongIterator(entriesInMemtableItr, entriesFromILSItr);
     }
+
+    @Override
+    public void setLimboState(long ledgerId) throws IOException {
+        throw new UnsupportedOperationException(
+                "Limbo state only supported for DbLedgerStorage");
+    }
+
+    @Override
+    public boolean hasLimboState(long ledgerId) throws IOException {
+        throw new UnsupportedOperationException(
+                "Limbo state only supported for DbLedgerStorage");
+    }
+
+    @Override
+    public void clearLimboState(long ledgerId) throws IOException {
+        throw new UnsupportedOperationException(
+                "Limbo state only supported for DbLedgerStorage");
+    }
+
+    @Override
+    public EnumSet<StorageState> getStorageStateFlags() throws IOException {
+        return EnumSet.noneOf(StorageState.class);
+    }
+
+    @Override
+    public void setStorageStateFlag(StorageState flags) throws IOException {
+        throw new UnsupportedOperationException(
+                "Storage state only flags supported for DbLedgerStorage");
+    }
+
+    @Override
+    public void clearStorageStateFlag(StorageState flags) throws IOException {
+        throw new UnsupportedOperationException(
+                "Storage state flags only supported for DbLedgerStorage");
+    }
 }
