@@ -32,7 +32,8 @@ import org.apache.bookkeeper.stats.StatsLogger;
 import org.apache.bookkeeper.stats.annotations.StatsDoc;
 
 /**
- * A umbrella class for db ledger storage stats.
+ * A umbrella class for db ledger storage stats with one instance per
+ * ledger directory.
  */
 @StatsDoc(
     name = BOOKIE_SCOPE,
@@ -46,11 +47,17 @@ class DbLedgerStorageStats {
     private static final String READ_ENTRY = "read-entry";
     private static final String READ_ENTRY_LOCATIONS_INDEX_TIME = "read-locations-index-time";
     private static final String READ_ENTRYLOG_TIME = "read-entrylog-time";
+    private static final String WRITE_CACHE_HITS = "write-cache-hits";
+    private static final String WRITE_CACHE_MISSES = "write-cache-misses";
     private static final String READ_CACHE_HITS = "read-cache-hits";
     private static final String READ_CACHE_MISSES = "read-cache-misses";
     private static final String READAHEAD_BATCH_COUNT = "readahead-batch-count";
     private static final String READAHEAD_BATCH_SIZE = "readahead-batch-size";
+    private static final String READAHEAD_TIME = "readahead-time";
     private static final String FLUSH = "flush";
+    private static final String FLUSH_ENTRYLOG = "flush-entrylog";
+    private static final String FLUSH_LOCATIONS_INDEX = "flush-locations-index";
+    private static final String FLUSH_LEDGER_INDEX = "flush-ledger-index";
     private static final String FLUSH_SIZE = "flush-size";
 
     @Deprecated
@@ -75,32 +82,42 @@ class DbLedgerStorageStats {
         parent = BOOKIE_ADD_ENTRY
     )
     private final OpStatsLogger readEntryStats;
-
     @StatsDoc(
             name = READ_ENTRY_LOCATIONS_INDEX_TIME,
             help = "time spent reading entries from the locations index of the db ledger storage engine",
             parent = READ_ENTRY
     )
-    private final OpStatsLogger readFromLocationIndexTime;
-
+    private final Counter readFromLocationIndexTime;
     @StatsDoc(
-        name = READ_ENTRYLOG_TIME,
-        help = "time spent reading entries from the entry log files of the db ledger storage engine",
-        parent = READ_ENTRY
+            name = READ_ENTRYLOG_TIME,
+            help = "time spent reading entries from the entry log files of the db ledger storage engine",
+            parent = READ_ENTRY
     )
-    private final OpStatsLogger readFromEntryLogTime;
+    private final Counter readFromEntryLogTime;
+    @StatsDoc(
+            name = WRITE_CACHE_HITS,
+            help = "number of write cache hits (on reads)",
+            parent = READ_ENTRY
+    )
+    private final Counter writeCacheHitCounter;
+    @StatsDoc(
+            name = WRITE_CACHE_MISSES,
+            help = "number of write cache misses (on reads)",
+            parent = READ_ENTRY
+    )
+    private final Counter writeCacheMissCounter;
     @StatsDoc(
         name = READ_CACHE_HITS,
-        help = "operation stats of read cache hits",
+        help = "number of read cache hits",
         parent = READ_ENTRY
     )
-    private final OpStatsLogger readCacheHitStats;
+    private final Counter readCacheHitCounter;
     @StatsDoc(
         name = READ_CACHE_MISSES,
-        help = "operation stats of read cache misses",
+        help = "number of read cache misses",
         parent = READ_ENTRY
     )
-    private final OpStatsLogger readCacheMissStats;
+    private final Counter readCacheMissCounter;
     @StatsDoc(
         name = READAHEAD_BATCH_COUNT,
         help = "the distribution of num of entries to read in one readahead batch"
@@ -111,11 +128,31 @@ class DbLedgerStorageStats {
         help = "the distribution of num of bytes to read in one readahead batch"
     )
     private final OpStatsLogger readAheadBatchSizeStats;
+    @StatsDoc(
+            name = READAHEAD_TIME,
+            help = "Time spent on readahead operations"
+    )
+    private final Counter readAheadTime;
     @StatsDoc(
         name = FLUSH,
         help = "operation stats of flushing write cache to entry log files"
     )
     private final OpStatsLogger flushStats;
+    @StatsDoc(
+            name = FLUSH_ENTRYLOG,
+            help = "operation stats of flushing to the current entry log file"
+    )
+    private final OpStatsLogger flushEntryLogStats;
+    @StatsDoc(
+            name = FLUSH_LOCATIONS_INDEX,
+            help = "operation stats of flushing to the locations index"
+    )
+    private final OpStatsLogger flushLocationIndexStats;
+    @StatsDoc(
+            name = FLUSH_LOCATIONS_INDEX,
+            help = "operation stats of flushing to the ledger index"
+    )
+    private final OpStatsLogger flushLedgerIndexStats;
     @StatsDoc(
         name = FLUSH_SIZE,
         help = "the distribution of number of bytes flushed from write cache to entry log files"
@@ -163,20 +200,26 @@ class DbLedgerStorageStats {
                          Supplier<Long> writeCacheCountSupplier,
                          Supplier<Long> readCacheSizeSupplier,
                          Supplier<Long> readCacheCountSupplier) {
-        addEntryStats = stats.getOpStatsLogger(ADD_ENTRY);
-        readEntryStats = stats.getOpStatsLogger(READ_ENTRY);
-        readFromLocationIndexTime = stats.getOpStatsLogger(READ_ENTRY_LOCATIONS_INDEX_TIME);
-        readFromEntryLogTime = stats.getOpStatsLogger(READ_ENTRYLOG_TIME);
-        readCacheHitStats = stats.getOpStatsLogger(READ_CACHE_HITS);
-        readCacheMissStats = stats.getOpStatsLogger(READ_CACHE_MISSES);
+        addEntryStats = stats.getThreadScopedOpStatsLogger(ADD_ENTRY);
+        readEntryStats = stats.getThreadScopedOpStatsLogger(READ_ENTRY);
+        readFromLocationIndexTime = stats.getThreadScopedCounter(READ_ENTRY_LOCATIONS_INDEX_TIME);
+        readFromEntryLogTime = stats.getThreadScopedCounter(READ_ENTRYLOG_TIME);
+        readCacheHitCounter = stats.getCounter(READ_CACHE_HITS);
+        readCacheMissCounter = stats.getCounter(READ_CACHE_MISSES);
+        writeCacheHitCounter = stats.getCounter(WRITE_CACHE_HITS);
+        writeCacheMissCounter = stats.getCounter(WRITE_CACHE_MISSES);
         readAheadBatchCountStats = stats.getOpStatsLogger(READAHEAD_BATCH_COUNT);
         readAheadBatchSizeStats = stats.getOpStatsLogger(READAHEAD_BATCH_SIZE);
+        readAheadTime = stats.getThreadScopedCounter(READAHEAD_TIME);
         flushStats = stats.getOpStatsLogger(FLUSH);
+        flushEntryLogStats = stats.getOpStatsLogger(FLUSH_ENTRYLOG);
+        flushLocationIndexStats = stats.getOpStatsLogger(FLUSH_LOCATIONS_INDEX);
+        flushLedgerIndexStats = stats.getOpStatsLogger(FLUSH_LEDGER_INDEX);
         flushSizeStats = stats.getOpStatsLogger(FLUSH_SIZE);
 
-        throttledWriteRequests = stats.getCounter(THROTTLED_WRITE_REQUESTS);
+        throttledWriteRequests = stats.getThreadScopedCounter(THROTTLED_WRITE_REQUESTS);
         throttledWriteStats = stats.getOpStatsLogger(THROTTLED_WRITE);
-        rejectedWriteRequests = stats.getCounter(REJECTED_WRITE_REQUESTS);
+        rejectedWriteRequests = stats.getThreadScopedCounter(REJECTED_WRITE_REQUESTS);
 
         writeCacheSizeGauge = new Gauge<Long>() {
             @Override
