@@ -33,7 +33,7 @@ import java.util.Date;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
 
-import org.apache.bookkeeper.bookie.Bookie;
+import org.apache.bookkeeper.bookie.BookieImpl;
 import org.apache.bookkeeper.bookie.EntryLogger;
 import org.apache.bookkeeper.bookie.EntryLogger.EntryLogScanner;
 import org.apache.bookkeeper.bookie.LedgerDirsManager;
@@ -46,7 +46,7 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
- * Scan all entries in the entry log and rebuild the ledgerStorageIndex.
+ * Scan all entries in the entry log and rebuild the locations index.
  */
 public class LocationsIndexRebuildOp {
     private final ServerConfiguration conf;
@@ -56,10 +56,10 @@ public class LocationsIndexRebuildOp {
     }
 
     public void initiate() throws IOException {
-        LOG.info("Starting index rebuilding");
+        LOG.info("Starting locations index rebuilding");
 
         // Move locations index to a backup directory
-        String basePath = Bookie.getCurrentDirectory(conf.getLedgerDirs()[0]).toString();
+        String basePath = BookieImpl.getCurrentDirectory(conf.getLedgerDirs()[0]).toString();
         Path currentPath = FileSystems.getDefault().getPath(basePath, "locations");
         String timestamp = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ").format(new Date());
         Path backupPath = FileSystems.getDefault().getPath(basePath, "locations.BACKUP-" + timestamp);
@@ -77,7 +77,7 @@ public class LocationsIndexRebuildOp {
         LOG.info("Found {} active ledgers in ledger manager", activeLedgers.size());
 
         KeyValueStorage newIndex = KeyValueStorageRocksDB.factory.newKeyValueStorage(basePath, "locations",
-                DbConfigType.Huge, conf);
+                DbConfigType.Default, conf);
 
         int totalEntryLogs = entryLogs.size();
         int completedEntryLogs = 0;
