@@ -289,7 +289,7 @@ public abstract class BKException extends org.apache.bookkeeper.client.api.BKExc
      */
     public static class BKNoSuchLedgerExistsOnMetadataServerException extends BKException {
         public BKNoSuchLedgerExistsOnMetadataServerException() {
-            super(Code.NoSuchLedgerExistsOnMetadataServerException);
+            super(BKException.Code.NoSuchLedgerExistsOnMetadataServerException);
         }
     }
 
@@ -467,6 +467,15 @@ public abstract class BKException extends org.apache.bookkeeper.client.api.BKExc
         }
     }
 
+    /**
+     * Bookkeeper ledger in limbo and data may or may not exist.
+     */
+    public static class BKDataUnknownException extends BKException {
+        public BKDataUnknownException() {
+            super(BKException.Code.DataUnknownException);
+        }
+    }
+
     /**
      * Extract an exception code from an BKException, or use a default if it's another type.
      * The throwable is null, assume that no exception took place and return
