@@ -23,12 +23,11 @@ package org.apache.bookkeeper.client;
 import static com.google.common.base.Preconditions.checkArgument;
 import static org.apache.bookkeeper.meta.MetadataDrivers.runFunctionWithMetadataBookieDriver;
 import static org.apache.bookkeeper.meta.MetadataDrivers.runFunctionWithRegistrationManager;
-
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
 import com.google.common.util.concurrent.UncheckedExecutionException;
-
+import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
@@ -51,11 +50,11 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.BiConsumer;
+import java.util.function.Function;
 import java.util.function.Predicate;
 import lombok.SneakyThrows;
-
-import org.apache.bookkeeper.bookie.Bookie;
 import org.apache.bookkeeper.bookie.BookieException;
+import org.apache.bookkeeper.bookie.BookieImpl;
 import org.apache.bookkeeper.client.AsyncCallback.OpenCallback;
 import org.apache.bookkeeper.client.AsyncCallback.RecoverCallback;
 import org.apache.bookkeeper.client.EnsemblePlacementPolicy.PlacementPolicyAdherence;
@@ -68,11 +67,13 @@ import org.apache.bookkeeper.conf.ClientConfiguration;
 import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.discover.BookieServiceInfo;
 import org.apache.bookkeeper.discover.RegistrationClient.RegistrationListener;
+import org.apache.bookkeeper.discover.RegistrationManager;
 import org.apache.bookkeeper.meta.LedgerAuditorManager;
 import org.apache.bookkeeper.meta.LedgerManager;
 import org.apache.bookkeeper.meta.LedgerManager.LedgerRangeIterator;
 import org.apache.bookkeeper.meta.LedgerManagerFactory;
 import org.apache.bookkeeper.meta.LedgerUnderreplicationManager;
+import org.apache.bookkeeper.meta.MetadataBookieDriver;
 import org.apache.bookkeeper.meta.UnderreplicatedLedger;
 import org.apache.bookkeeper.meta.zk.ZKMetadataDriverBase;
 import org.apache.bookkeeper.net.BookieId;
@@ -246,10 +247,10 @@ public class BookKeeperAdmin implements AutoCloseable {
     }
 
     @SneakyThrows
-    public BookieServiceInfo getBookieServiceInfo(String bookiedId)
+    public BookieServiceInfo getBookieServiceInfo(BookieId bookiedId)
             throws BKException {
         return FutureUtils.result(bkc.getMetadataClientDriver()
-                .getRegistrationClient().getBookieServiceInfo(BookieId.parse(bookiedId))).getValue();
+                .getRegistrationClient().getBookieServiceInfo(bookiedId)).getValue();
     }
 
     /**
@@ -431,12 +432,11 @@ public class BookKeeperAdmin implements AutoCloseable {
             if (currentEntry != null) {
                 return true;
             }
-            if (lastEntryId == -1 || nextEntryId <= lastEntryId) {
+            if ((lastEntryId == -1 || nextEntryId <= lastEntryId) && nextEntryId <= handle.getLastAddConfirmed()) {
                 try {
                     CompletableFuture<Enumeration<LedgerEntry>> result = new CompletableFuture<>();
-
                     handle.asyncReadEntriesInternal(nextEntryId, nextEntryId,
-                                                    new SyncReadCallback(result), null, false);
+                        new SyncReadCallback(result), null, false);
 
                     currentEntry = SyncCallbackUtils.waitForResult(result).nextElement();
 
@@ -1240,36 +1240,42 @@ public class BookKeeperAdmin implements AutoCloseable {
      */
     public static boolean format(ServerConfiguration conf,
             boolean isInteractive, boolean force) throws Exception {
-        return runFunctionWithMetadataBookieDriver(conf, driver -> {
-            try {
-                boolean ledgerRootExists = driver.getRegistrationManager().prepareFormat();
+        return runFunctionWithMetadataBookieDriver(conf, new Function<MetadataBookieDriver, Boolean>() {
+            @Override
+            @SuppressFBWarnings("RCN_REDUNDANT_NULLCHECK_WOULD_HAVE_BEEN_A_NPE")
+            public Boolean apply(MetadataBookieDriver driver) {
+                try {
+                    try (RegistrationManager regManager = driver.createRegistrationManager()) {
+                        boolean ledgerRootExists = regManager.prepareFormat();
 
-                // If old data was there then confirm with admin.
-                boolean doFormat = true;
-                if (ledgerRootExists) {
-                    if (!isInteractive) {
-                        // If non interactive and force is set, then delete old data.
-                        doFormat = force;
-                    } else {
-                        // Confirm with the admin.
-                        doFormat = IOUtils
-                            .confirmPrompt("Ledger root already exists. "
-                                + "Are you sure to format bookkeeper metadata? "
-                                + "This may cause data loss.");
+                        // If old data was there then confirm with admin.
+                        boolean doFormat = true;
+                        if (ledgerRootExists) {
+                            if (!isInteractive) {
+                                // If non interactive and force is set, then delete old data.
+                                doFormat = force;
+                            } else {
+                                // Confirm with the admin.
+                                doFormat = IOUtils
+                                        .confirmPrompt("Ledger root already exists. "
+                                                + "Are you sure to format bookkeeper metadata? "
+                                                + "This may cause data loss.");
+                            }
+                        }
+
+                        if (!doFormat) {
+                            return false;
+                        }
+
+                        driver.getLedgerManagerFactory().format(
+                                conf,
+                                driver.getLayoutManager());
+
+                        return regManager.format();
                     }
+                } catch (Exception e) {
+                    throw new UncheckedExecutionException(e.getMessage(), e);
                 }
-
-                if (!doFormat) {
-                    return false;
-                }
-
-                driver.getLedgerManagerFactory().format(
-                    conf,
-                    driver.getLayoutManager());
-
-                return driver.getRegistrationManager().format();
-            } catch (Exception e) {
-                throw new UncheckedExecutionException(e.getMessage(), e);
             }
         });
     }
@@ -1366,7 +1372,7 @@ public class BookKeeperAdmin implements AutoCloseable {
                  * make sure that there is no bookie registered with the same
                  * bookieid and the cookie for the same bookieid is not existing.
                  */
-                BookieId bookieId = Bookie.getBookieId(conf);
+                BookieId bookieId = BookieImpl.getBookieId(conf);
                 if (rm.isBookieRegistered(bookieId)) {
                     LOG.error("Bookie with bookieId: {} is still registered, "
                         + "If this node is running bookie process, try stopping it first.", bookieId);
