@@ -118,7 +118,7 @@ class ForceLedgerOp extends SafeRunnable implements ForceLedgerCallback {
         } else {
             // at least one bookie failed, as we are waiting for all the bookies
             // we can fail immediately
-            LOG.info("ForceLedger did not succeed: Ledger {} on {}", ledgerId, addr);
+            LOG.error("ForceLedger did not succeed: Ledger {} on {}", ledgerId, addr);
             errored = true;
 
             // notify the failure
