@@ -24,8 +24,8 @@ import static org.apache.bookkeeper.replication.ReplicationStats.NUM_BYTES_READ;
 import static org.apache.bookkeeper.replication.ReplicationStats.NUM_BYTES_WRITTEN;
 import static org.apache.bookkeeper.replication.ReplicationStats.NUM_ENTRIES_READ;
 import static org.apache.bookkeeper.replication.ReplicationStats.NUM_ENTRIES_WRITTEN;
-import static org.apache.bookkeeper.replication.ReplicationStats.READ_DATA_LATENCY;;
-import static org.apache.bookkeeper.replication.ReplicationStats.REPLICATION_WORKER_SCOPE;;
+import static org.apache.bookkeeper.replication.ReplicationStats.READ_DATA_LATENCY;
+import static org.apache.bookkeeper.replication.ReplicationStats.REPLICATION_WORKER_SCOPE;
 import static org.apache.bookkeeper.replication.ReplicationStats.WRITE_DATA_LATENCY;
 import com.google.common.util.concurrent.RateLimiter;
 import io.netty.buffer.Unpooled;
@@ -285,7 +285,7 @@ public class LedgerFragmentReplicator {
             assert false;
         }
 
-        long numberOfEntriesToReplicate = (lastEntryId - firstEntryId) + 1;
+        long numberOfEntriesToReplicate = firstEntryId == INVALID_ENTRY_ID ? 0 : (lastEntryId - firstEntryId) + 1;
         long splitsWithFullEntries = numberOfEntriesToReplicate
                 / rereplicationEntryBatchSize;
 
@@ -420,7 +420,7 @@ public class LedgerFragmentReplicator {
 
     private void updateAverageEntrySize(int toSendSize) {
         averageEntrySize.updateAndGet(value -> (int) (value * AVERAGE_ENTRY_SIZE_RATIO
-            + (1 - AVERAGE_ENTRY_SIZE_RATIO) * toSendSize));
+                + (1 - AVERAGE_ENTRY_SIZE_RATIO) * toSendSize));
     }
 
     /**
