@@ -40,6 +40,7 @@ import org.apache.bookkeeper.client.impl.OpenBuilderBase;
 import org.apache.bookkeeper.stats.OpStatsLogger;
 import org.apache.bookkeeper.util.MathUtils;
 import org.apache.bookkeeper.util.OrderedGenericCallback;
+import org.apache.bookkeeper.util.SafeRunnable;
 import org.apache.bookkeeper.versioning.Versioned;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -112,12 +113,10 @@ class LedgerOpenOp {
          * Asynchronously read the ledger metadata node.
          */
         bk.getLedgerManager().readLedgerMetadata(ledgerId)
-            .whenComplete((metadata, exception) -> {
-                    if (exception != null) {
-                        openComplete(BKException.getExceptionCode(exception), null);
-                    } else {
-                        openWithMetadata(metadata);
-                    }
+                .thenAcceptAsync(this::openWithMetadata, bk.getScheduler().chooseThread(ledgerId))
+                .exceptionally(exception -> {
+                    openComplete(BKException.getExceptionCode(exception), null);
+                    return null;
                 });
     }
 
@@ -129,17 +128,11 @@ class LedgerOpenOp {
         initiate();
     }
 
-    private void closeLedgerHandle() {
-        try {
-            if (lh != null) {
-                lh.close();
-            }
-        } catch (InterruptedException e) {
-            Thread.currentThread().interrupt();
-            LOG.info("InterruptedException while closing ledger {}", ledgerId, e);
-        } catch (BKException e) {
-            LOG.warn("BKException while closing ledger {} ", ledgerId, e);
+    private CompletableFuture<Void> closeLedgerHandleAsync() {
+        if (lh != null) {
+            return lh.closeAsync();
         }
+        return CompletableFuture.completedFuture(null);
     }
 
     private void openWithMetadata(Versioned<LedgerMetadata> versionedMetadata) {
@@ -208,12 +201,18 @@ class LedgerOpenOp {
                 public void safeOperationComplete(int rc, Void result) {
                     if (rc == BKException.Code.OK) {
                         openComplete(BKException.Code.OK, lh);
-                    } else if (rc == BKException.Code.UnauthorizedAccessException) {
-                        closeLedgerHandle();
-                        openComplete(BKException.Code.UnauthorizedAccessException, null);
                     } else {
-                        closeLedgerHandle();
-                        openComplete(bk.getReturnRc(BKException.Code.LedgerRecoveryException), null);
+                        closeLedgerHandleAsync().whenComplete((ignore, ex) -> {
+                            if (ex != null) {
+                                LOG.error("Ledger {} close failed", ledgerId, ex);
+                            }
+                            if (rc == BKException.Code.UnauthorizedAccessException
+                                    || rc == BKException.Code.TimeoutException) {
+                                openComplete(bk.getReturnRc(rc), null);
+                            } else {
+                                openComplete(bk.getReturnRc(BKException.Code.LedgerRecoveryException), null);
+                            }
+                        });
                     }
                 }
                 @Override
@@ -226,9 +225,20 @@ class LedgerOpenOp {
                 @Override
                 public void readLastConfirmedComplete(int rc,
                         long lastConfirmed, Object ctx) {
-                    if (rc != BKException.Code.OK) {
-                        closeLedgerHandle();
-                        openComplete(bk.getReturnRc(BKException.Code.ReadException), null);
+                    if (rc == BKException.Code.TimeoutException) {
+                        closeLedgerHandleAsync().whenComplete((r, ex) -> {
+                            if (ex != null) {
+                                LOG.error("Ledger {} close failed", ledgerId, ex);
+                            }
+                            openComplete(bk.getReturnRc(rc), null);
+                        });
+                    } else if (rc != BKException.Code.OK) {
+                        closeLedgerHandleAsync().whenComplete((r, ex) -> {
+                            if (ex != null) {
+                                LOG.error("Ledger {} close failed", ledgerId, ex);
+                            }
+                            openComplete(bk.getReturnRc(BKException.Code.ReadException), null);
+                        });
                     } else {
                         lh.lastAddConfirmed = lh.lastAddPushed = lastConfirmed;
                         openComplete(BKException.Code.OK, lh);
@@ -245,7 +255,17 @@ class LedgerOpenOp {
         } else {
             openOpLogger.registerSuccessfulEvent(MathUtils.elapsedNanos(startTime), TimeUnit.NANOSECONDS);
         }
-        cb.openComplete(rc, lh, ctx);
+
+        if (lh != null) { // lh is null in case of errors
+            lh.executeOrdered(new SafeRunnable() {
+                @Override
+                public void safeRun() {
+                    cb.openComplete(rc, lh, ctx);
+                }
+            });
+        } else {
+            cb.openComplete(rc, null, ctx);
+        }
     }
 
     static final class OpenBuilderImpl extends OpenBuilderBase {
