@@ -156,7 +156,9 @@ class PendingReadLacOp implements ReadLacCallback {
         }
 
         if (numResponsesPending == 0 && !completed) {
-            LOG.info("While readLac ledger: " + ledgerId + " did not hear success responses from all of ensemble");
+            LOG.error(
+                    "While readLac ledger: {} did not hear success responses from all of ensemble, coverageSet is: {}",
+                    ledgerId, coverageSet);
             cb.getLacComplete(lastSeenError, maxLac);
         }
     }
