@@ -236,6 +236,16 @@ public interface LedgerUnderreplicationManager extends AutoCloseable {
      */
     long getReplicasCheckCTime() throws ReplicationException.UnavailableException;
 
+    /**
+     * Receive notification asynchronously when the num of under-replicated ledgers  Changed.
+     *
+     * @param cb
+     * @throws ReplicationException.UnavailableException
+     */
+    @Deprecated
+    default void notifyUnderReplicationLedgerChanged(GenericCallback<Void> cb)
+        throws ReplicationException.UnavailableException {}
+
     /**
      * Receive notification asynchronously when the lostBookieRecoveryDelay value is Changed.
      *
