@@ -167,11 +167,10 @@ public class BookieSocketAddress {
 
     /**
      * Use legacy resolver to resolve a bookieId.
-     * @param bookieId id supposed to be generated by
-     * {@link #createDummyBookieIdForHostname(java.lang.String)}
+     * @param bookieId legacy style bookie ID consisting of address (or hostname) and port
      * @return the BookieSocketAddress
      */
-    public static BookieSocketAddress resolveDummyBookieId(BookieId bookieId)
+    public static BookieSocketAddress resolveLegacyBookieId(BookieId bookieId)
             throws BookieAddressResolver.BookieIdNotResolvedException {
         return LEGACY_BOOKIEID_RESOLVER.resolve(bookieId);
     }
