@@ -66,6 +66,7 @@ class ReadEntryProcessor extends PacketProcessorBase<ReadRequest> {
                 LOG.debug("Dropping read request for closed channel: {}", channel);
             }
             requestProcessor.onReadRequestFinish();
+            recycle();
             return;
         }
         int errorCode = BookieProtocol.EOK;
@@ -188,7 +189,8 @@ class ReadEntryProcessor extends PacketProcessorBase<ReadRequest> {
         return String.format("ReadEntry(%d, %d)", request.getLedgerId(), request.getEntryId());
     }
 
-    private void recycle() {
+    void recycle() {
+        request.recycle();
         super.reset();
         this.recyclerHandle.recycle(this);
     }
