@@ -27,6 +27,7 @@ import java.io.File;
 import java.io.IOException;
 import java.lang.Thread.UncaughtExceptionHandler;
 import java.net.MalformedURLException;
+import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutionException;
 import org.apache.bookkeeper.bookie.BookieCriticalThread;
 import org.apache.bookkeeper.bookie.BookieImpl;
@@ -93,7 +94,11 @@ public class AutoRecoveryMain {
         MetadataClientDriver metadataClientDriver = bkc.getMetadataClientDriver();
         metadataClientDriver.setSessionStateListener(() -> {
             LOG.error("Client connection to the Metadata server has expired, so shutting down AutoRecoveryMain!");
-            shutdown(ExitCode.ZK_EXPIRED);
+            // do not run "shutdown" in the main ZooKeeper client thread
+            // as it performs some blocking operations
+            CompletableFuture.runAsync(() -> {
+                shutdown(ExitCode.ZK_EXPIRED);
+            });
         });
 
         auditorElector = new AuditorElector(
