@@ -20,7 +20,6 @@ package org.apache.bookkeeper.tools.cli.commands.cookie;
 
 import static org.apache.bookkeeper.meta.MetadataDrivers.runFunctionWithMetadataBookieDriver;
 import static org.apache.bookkeeper.meta.MetadataDrivers.runFunctionWithRegistrationManager;
-
 import com.beust.jcommander.Parameter;
 import com.google.common.collect.Lists;
 import com.google.common.util.concurrent.UncheckedExecutionException;
@@ -31,10 +30,12 @@ import java.util.LinkedList;
 import java.util.List;
 import lombok.Setter;
 import lombok.experimental.Accessors;
-import org.apache.bookkeeper.bookie.Bookie;
 import org.apache.bookkeeper.bookie.BookieException;
+import org.apache.bookkeeper.bookie.BookieImpl;
 import org.apache.bookkeeper.bookie.Cookie;
+import org.apache.bookkeeper.bookie.LegacyCookieValidation;
 import org.apache.bookkeeper.conf.ServerConfiguration;
+import org.apache.bookkeeper.discover.RegistrationManager;
 import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.tools.cli.helpers.BookieCommand;
 import org.apache.bookkeeper.tools.framework.CliFlags;
@@ -110,12 +111,12 @@ public class AdminCommand extends BookieCommand<AdminCommand.AdminFlags> {
     }
 
     private void initDirectory(ServerConfiguration bkConf) {
-        this.journalDirectories = Bookie.getCurrentDirectories(bkConf.getJournalDirs());
-        this.ledgerDirectories = Bookie.getCurrentDirectories(bkConf.getLedgerDirs());
+        this.journalDirectories = BookieImpl.getCurrentDirectories(bkConf.getJournalDirs());
+        this.ledgerDirectories = BookieImpl.getCurrentDirectories(bkConf.getLedgerDirs());
         if (null == bkConf.getIndexDirs()) {
             this.indexDirectories = this.ledgerDirectories;
         } else {
-            this.indexDirectories = Bookie.getCurrentDirectories(bkConf.getIndexDirs());
+            this.indexDirectories = BookieImpl.getCurrentDirectories(bkConf.getIndexDirs());
         }
     }
 
@@ -149,7 +150,7 @@ public class AdminCommand extends BookieCommand<AdminCommand.AdminFlags> {
         return runFunctionWithRegistrationManager(bkConf, rm -> {
             try {
                 ServerConfiguration conf = new ServerConfiguration(bkConf);
-                String newBookieId = Bookie.getBookieId(conf).toString();
+                String newBookieId = BookieImpl.getBookieId(conf).toString();
                 // read oldcookie
                 Versioned<Cookie> oldCookie = null;
                 try {
@@ -235,9 +236,12 @@ public class AdminCommand extends BookieCommand<AdminCommand.AdminFlags> {
                 allLedgerDirs.addAll(Arrays.asList(indexDirectories));
             }
 
-            try {
-                Bookie.checkEnvironmentWithStorageExpansion(bkConf, driver, Arrays.asList(journalDirectories),
-                                                            allLedgerDirs);
+            try (RegistrationManager registrationManager = driver.createRegistrationManager()) {
+                LegacyCookieValidation validation = new LegacyCookieValidation(bkConf, registrationManager);
+                List<File> dirs = Lists.newArrayList();
+                dirs.addAll(Arrays.asList(journalDirectories));
+                dirs.addAll(allLedgerDirs);
+                validation.checkCookies(dirs);
                 return true;
             } catch (BookieException e) {
                 LOG.error("Exception while updating cookie for storage expansion", e);
@@ -247,7 +251,7 @@ public class AdminCommand extends BookieCommand<AdminCommand.AdminFlags> {
     }
 
     private boolean listOrDeleteCookies(ServerConfiguration bkConf, boolean delete, boolean force) throws Exception {
-        BookieId bookieAddress = Bookie.getBookieId(bkConf);
+        BookieId bookieAddress = BookieImpl.getBookieId(bkConf);
         File[] journalDirs = bkConf.getJournalDirs();
         File[] ledgerDirs = bkConf.getLedgerDirs();
         File[] indexDirs = bkConf.getIndexDirs();
@@ -256,7 +260,7 @@ public class AdminCommand extends BookieCommand<AdminCommand.AdminFlags> {
             allDirs = ArrayUtils.addAll(allDirs, indexDirs);
         }
 
-        File[] allCurDirs = Bookie.getCurrentDirectories(allDirs);
+        File[] allCurDirs = BookieImpl.getCurrentDirectories(allDirs);
         List<File> allVersionFiles = new LinkedList<File>();
         File versionFile;
         for (File curDir : allCurDirs) {
