@@ -18,7 +18,7 @@
 package org.apache.bookkeeper.verifier;
 
 import static com.google.common.base.Preconditions.checkState;
-
+import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -32,7 +32,6 @@ import java.util.Set;
 import java.util.TreeSet;
 import java.util.function.BiConsumer;
 import java.util.function.Consumer;
-
 import org.apache.bookkeeper.client.BKException;
 
 /**
@@ -180,6 +179,7 @@ public class BookkeeperVerifier {
     /**
      * State required to regenerate an entry.
      */
+    @SuppressFBWarnings("DMI_RANDOM_USED_ONLY_ONCE")
     class EntryInfo {
         private final long entryID;
         private final long seed;
@@ -471,7 +471,7 @@ public class BookkeeperVerifier {
                                 current, ledger.ledgerID, result.length, check.length)
                         ));
                     }
-                        /* Verify contents */
+                    /* Verify contents */
                     if (!Arrays.equals(check, result)) {
                         int i = 0;
                         for (; i < check.length; ++i) {
@@ -657,7 +657,7 @@ public class BookkeeperVerifier {
 
         /* Wait for all in progress ops to complete, outstanding*Count is updated under the lock */
         while ((System.currentTimeMillis() < testDrainEnd)
-               && (outstandingReadCount > 0 || outstandingWriteCount > 0)) {
+                && (outstandingReadCount > 0 || outstandingWriteCount > 0)) {
             System.out.format("reads: %d, writes: %d%n", outstandingReadCount, outstandingWriteCount);
             System.out.format("openingLedgers:%n");
             for (LedgerInfo li: openingLedgers) {
