@@ -63,7 +63,7 @@ import org.powermock.reflect.Whitebox;
  * Test the bookie journal.
  */
 @RunWith(PowerMockRunner.class)
-@PrepareForTest({JournalChannel.class, Journal.class})
+@PrepareForTest({JournalChannel.class, Journal.class, DefaultFileChannel.class})
 @Slf4j
 public class BookieJournalForceTest {
 
@@ -75,7 +75,7 @@ public class BookieJournalForceTest {
     @Test
     public void testAckAfterSync() throws Exception {
         File journalDir = tempDir.newFolder();
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration()
             .setJournalDirName(journalDir.getPath())
@@ -138,7 +138,7 @@ public class BookieJournalForceTest {
     @Test
     public void testAckBeforeSync() throws Exception {
         File journalDir = tempDir.newFolder();
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
@@ -189,7 +189,7 @@ public class BookieJournalForceTest {
     @Test
     public void testAckBeforeSyncWithJournalBufferedEntriesThreshold() throws Exception {
         File journalDir = tempDir.newFolder();
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         final int journalBufferedEntriesThreshold = 10;
         // sending a burst of entries, more than journalBufferedEntriesThreshold
@@ -255,7 +255,7 @@ public class BookieJournalForceTest {
     @Test
     public void testInterleavedRequests() throws Exception {
         File journalDir = tempDir.newFolder();
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
@@ -322,7 +322,7 @@ public class BookieJournalForceTest {
     @Test
     public void testForceLedger() throws Exception {
         File journalDir = tempDir.newFolder();
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
 
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath());
@@ -379,4 +379,26 @@ public class BookieJournalForceTest {
         journal.shutdown();
     }
 
+    @Test
+    public void testFileChannelProvider() throws Exception {
+        File bookieFileDirectory = tempDir.newFile();
+        ServerConfiguration config = TestBKConfiguration.newServerConfiguration();
+
+        DefaultFileChannel defaultFileChannel = spy(new DefaultFileChannel(bookieFileDirectory, config));
+
+        FileChannelProvider provider = spy(DefaultFileChannelProvider.class);
+        when(provider.open(bookieFileDirectory, config)).thenReturn(defaultFileChannel);
+        log.info("Journal Channel Provider: " + config.getJournalChannelProvider());
+        // Open should return spied DefaultFileChannel here.
+        BookieFileChannel bookieFileChannel = provider.open(bookieFileDirectory, config);
+        bookieFileChannel.getFileChannel();
+        verify(defaultFileChannel, times (1)).getFileChannel();
+        bookieFileChannel.getFD();
+        verify(defaultFileChannel, times (1)).getFD();
+        bookieFileChannel.fileExists(bookieFileDirectory);
+        verify(defaultFileChannel, times (1)).fileExists(bookieFileDirectory);
+        provider.close(bookieFileChannel);
+        verify(defaultFileChannel, times (1)).close();
+    }
+
 }
