@@ -29,7 +29,6 @@ import static org.mockito.ArgumentMatchers.anyBoolean;
 import static org.mockito.Mockito.doAnswer;
 import static org.mockito.Mockito.mock;
 import static org.powermock.api.mockito.PowerMockito.whenNew;
-
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.Unpooled;
 import java.io.File;
@@ -43,6 +42,7 @@ import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.conf.TestBKConfiguration;
 import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.proto.BookkeeperInternalCallbacks.WriteCallback;
+import org.apache.bookkeeper.util.DiskChecker;
 import org.apache.commons.lang3.mutable.MutableBoolean;
 import org.junit.Rule;
 import org.junit.Test;
@@ -50,6 +50,7 @@ import org.junit.rules.TemporaryFolder;
 import org.junit.runner.RunWith;
 import org.mockito.invocation.InvocationOnMock;
 import org.mockito.stubbing.Answer;
+import org.powermock.core.classloader.annotations.PowerMockIgnore;
 import org.powermock.core.classloader.annotations.PrepareForTest;
 import org.powermock.modules.junit4.PowerMockRunner;
 
@@ -57,17 +58,18 @@ import org.powermock.modules.junit4.PowerMockRunner;
  * Test the bookie journal.
  */
 @RunWith(PowerMockRunner.class)
-@PrepareForTest({Bookie.class})
+@PrepareForTest({BookieImpl.class})
+@PowerMockIgnore({"jdk.internal.loader.*", "javax.naming.*"})
 @Slf4j
 public class BookieWriteToJournalTest {
 
     @Rule
     public TemporaryFolder tempDir = new TemporaryFolder();
 
-    class NoOpJournalReplayBookie extends Bookie {
+    class NoOpJournalReplayBookie extends TestBookieImpl {
 
         public NoOpJournalReplayBookie(ServerConfiguration conf)
-                throws IOException, InterruptedException, BookieException {
+                throws Exception {
             super(conf);
         }
 
@@ -80,18 +82,20 @@ public class BookieWriteToJournalTest {
     /**
      * test that Bookie calls correctly Journal.logAddEntry about "ackBeforeSync" parameter.
      */
+
     @Test
     public void testJournalLogAddEntryCalledCorrectly() throws Exception {
 
         File journalDir = tempDir.newFolder();
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
         File ledgerDir = tempDir.newFolder();
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
                 .setLedgerDirNames(new String[]{ledgerDir.getPath()})
                 .setMetadataServiceUri(null);
-        BookieId bookieAddress = Bookie.getBookieId(conf);
+
+        BookieId bookieAddress = BookieImpl.getBookieId(conf);
         CountDownLatch journalJoinLatch = new CountDownLatch(1);
         Journal journal = mock(Journal.class);
         MutableBoolean effectiveAckBeforeSync = new MutableBoolean(false);
@@ -150,14 +154,14 @@ public class BookieWriteToJournalTest {
     public void testForceLedger() throws Exception {
 
         File journalDir = tempDir.newFolder();
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(journalDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(journalDir));
         File ledgerDir = tempDir.newFolder();
-        Bookie.checkDirectoryStructure(Bookie.getCurrentDirectory(ledgerDir));
+        BookieImpl.checkDirectoryStructure(BookieImpl.getCurrentDirectory(ledgerDir));
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setJournalDirName(journalDir.getPath())
                 .setLedgerDirNames(new String[]{ledgerDir.getPath()});
 
-        Bookie b = new Bookie(conf);
+        Bookie b = new TestBookieImpl(conf);
         b.start();
 
         long ledgerId = 1;
@@ -204,6 +208,33 @@ public class BookieWriteToJournalTest {
         b.shutdown();
     }
 
+    @Test
+    public void testSmallJournalQueueWithHighFlushFrequency() throws IOException, InterruptedException {
+        ServerConfiguration conf = new ServerConfiguration();
+        conf.setJournalQueueSize(1);
+        conf.setJournalFlushWhenQueueEmpty(true);
+        conf.setJournalBufferedWritesThreshold(1);
+
+        conf.setJournalDirName(tempDir.newFolder().getPath());
+        conf.setLedgerDirNames(new String[]{tempDir.newFolder().getPath()});
+        DiskChecker diskChecker = new DiskChecker(conf.getDiskUsageThreshold(), conf.getDiskUsageWarnThreshold());
+        LedgerDirsManager ledgerDirsManager = new LedgerDirsManager(conf, conf.getLedgerDirs(), diskChecker);
+        Journal journal = new Journal(0, conf.getJournalDirs()[0], conf, ledgerDirsManager);
+        journal.start();
+
+        final int entries = 1000;
+        CountDownLatch entriesLatch = new CountDownLatch(entries);
+        for (int j = 1; j <= entries; j++) {
+            ByteBuf entry = buildEntry(1, j, -1);
+            journal.logAddEntry(entry, false, (int rc, long ledgerId, long entryId, BookieId addr, Object ctx) -> {
+                entriesLatch.countDown();
+            }, null);
+        }
+        entriesLatch.await();
+
+        journal.shutdown();
+    }
+
     private static ByteBuf buildEntry(long ledgerId, long entryId, long lastAddConfirmed) {
         final ByteBuf data = Unpooled.buffer();
         data.writeLong(ledgerId);
