@@ -23,6 +23,8 @@ package org.apache.bookkeeper.bookie;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyLong;
 
+import io.netty.buffer.PooledByteBufAllocator;
+
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
@@ -53,6 +55,7 @@ import org.apache.bookkeeper.conf.ClientConfiguration;
 import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.conf.TestBKConfiguration;
 import org.apache.bookkeeper.proto.BookieServer;
+import org.apache.bookkeeper.stats.NullStatsLogger;
 import org.apache.bookkeeper.test.ZooKeeperUtil;
 import org.apache.bookkeeper.util.IOUtils;
 import org.apache.bookkeeper.util.PortManager;
@@ -76,7 +79,7 @@ import org.slf4j.LoggerFactory;
  */
 @RunWith(PowerMockRunner.class)
 @PrepareForTest(SyncThread.class)
-@PowerMockIgnore("javax.*")
+@PowerMockIgnore({"java.*", "javax.*", "org.slf4j.*"})
 public class LedgerStorageCheckpointTest {
     private static final Logger LOG = LoggerFactory
             .getLogger(LedgerStorageCheckpointTest.class);
@@ -216,8 +219,11 @@ public class LedgerStorageCheckpointTest {
                 .setLedgerStorageClass(ledgerStorageClassName);
         Assert.assertEquals("Number of JournalDirs", 1, conf.getJournalDirs().length);
         // we know there is only one ledgerDir
-        File ledgerDir = Bookie.getCurrentDirectories(conf.getLedgerDirs())[0];
-        BookieServer server = new BookieServer(conf);
+        File ledgerDir = BookieImpl.getCurrentDirectories(conf.getLedgerDirs())[0];
+        BookieServer server = new BookieServer(
+                conf, new TestBookieImpl(conf),
+                NullStatsLogger.INSTANCE, PooledByteBufAllocator.DEFAULT,
+                new MockUncleanShutdownDetection());
         server.start();
         ClientConfiguration clientConf = new ClientConfiguration();
         clientConf.setMetadataServiceUri(zkUtil.getMetadataServiceUri());
@@ -237,7 +243,7 @@ public class LedgerStorageCheckpointTest {
             handle.close();
         }
 
-        LastLogMark lastLogMarkAfterFirstSetOfAdds = server.getBookie().journals.get(0).getLastLogMark();
+        LastLogMark lastLogMarkAfterFirstSetOfAdds = ((BookieImpl) server.getBookie()).journals.get(0).getLastLogMark();
         LogMark curMarkAfterFirstSetOfAdds = lastLogMarkAfterFirstSetOfAdds.getCurMark();
 
         File lastMarkFile = new File(ledgerDir, "lastMark");
@@ -258,7 +264,7 @@ public class LedgerStorageCheckpointTest {
         Assert.assertTrue("lastMark file must be existing, because checkpoint should have happened",
                 lastMarkFile.exists());
 
-        LastLogMark lastLogMarkAfterCheckpoint = server.getBookie().journals.get(0).getLastLogMark();
+        LastLogMark lastLogMarkAfterCheckpoint = ((BookieImpl) server.getBookie()).journals.get(0).getLastLogMark();
         LogMark curMarkAfterCheckpoint = lastLogMarkAfterCheckpoint.getCurMark();
 
         LogMark rolledLogMark = readLastMarkFile(lastMarkFile);
@@ -291,7 +297,8 @@ public class LedgerStorageCheckpointTest {
         // wait for flushInterval for SyncThread to do next iteration of checkpoint
         executorController.advance(Duration.ofMillis(conf.getFlushInterval()));
 
-        LastLogMark lastLogMarkAfterSecondSetOfAdds = server.getBookie().journals.get(0).getLastLogMark();
+        LastLogMark lastLogMarkAfterSecondSetOfAdds = ((BookieImpl) server.getBookie()).
+                journals.get(0).getLastLogMark();
         LogMark curMarkAfterSecondSetOfAdds = lastLogMarkAfterSecondSetOfAdds.getCurMark();
 
         rolledLogMark = readLastMarkFile(lastMarkFile);
@@ -344,13 +351,16 @@ public class LedgerStorageCheckpointTest {
 
         Assert.assertEquals("Number of JournalDirs", 1, conf.getJournalDirs().length);
         // we know there is only one ledgerDir
-        File ledgerDir = Bookie.getCurrentDirectories(conf.getLedgerDirs())[0];
-        BookieServer server = new BookieServer(conf);
+        File ledgerDir = BookieImpl.getCurrentDirectories(conf.getLedgerDirs())[0];
+        BookieServer server = new BookieServer(
+                conf, new TestBookieImpl(conf),
+                NullStatsLogger.INSTANCE, PooledByteBufAllocator.DEFAULT,
+                new MockUncleanShutdownDetection());
         server.start();
         ClientConfiguration clientConf = new ClientConfiguration();
         clientConf.setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         BookKeeper bkClient = new BookKeeper(clientConf);
-        InterleavedLedgerStorage ledgerStorage = (InterleavedLedgerStorage) server.getBookie().ledgerStorage;
+        InterleavedLedgerStorage ledgerStorage = (InterleavedLedgerStorage) server.getBookie().getLedgerStorage();
 
         int numOfEntries = 5;
         byte[] dataBytes = "data".getBytes();
@@ -422,8 +432,11 @@ public class LedgerStorageCheckpointTest {
 
         Assert.assertEquals("Number of JournalDirs", 1, conf.getJournalDirs().length);
         // we know there is only one ledgerDir
-        File ledgerDir = Bookie.getCurrentDirectories(conf.getLedgerDirs())[0];
-        BookieServer server = new BookieServer(conf);
+        File ledgerDir = BookieImpl.getCurrentDirectories(conf.getLedgerDirs())[0];
+        BookieServer server = new BookieServer(
+                conf, new TestBookieImpl(conf),
+                NullStatsLogger.INSTANCE, PooledByteBufAllocator.DEFAULT,
+                new MockUncleanShutdownDetection());
         server.start();
         ClientConfiguration clientConf = new ClientConfiguration();
         clientConf.setMetadataServiceUri(zkUtil.getMetadataServiceUri());
@@ -485,13 +498,16 @@ public class LedgerStorageCheckpointTest {
 
         Assert.assertEquals("Number of JournalDirs", 1, conf.getJournalDirs().length);
         // we know there is only one ledgerDir
-        File ledgerDir = Bookie.getCurrentDirectories(conf.getLedgerDirs())[0];
-        BookieServer server = new BookieServer(conf);
+        File ledgerDir = BookieImpl.getCurrentDirectories(conf.getLedgerDirs())[0];
+        BookieServer server = new BookieServer(
+                conf, new TestBookieImpl(conf),
+                NullStatsLogger.INSTANCE, PooledByteBufAllocator.DEFAULT,
+                new MockUncleanShutdownDetection());
         server.start();
         ClientConfiguration clientConf = new ClientConfiguration();
         clientConf.setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         BookKeeper bkClient = new BookKeeper(clientConf);
-        InterleavedLedgerStorage ledgerStorage = (InterleavedLedgerStorage) server.getBookie().ledgerStorage;
+        InterleavedLedgerStorage ledgerStorage = (InterleavedLedgerStorage) server.getBookie().getLedgerStorage();
         EntryLogger entryLogger = ledgerStorage.entryLogger;
         EntryLogManagerForEntryLogPerLedger entryLogManager = (EntryLogManagerForEntryLogPerLedger) entryLogger
                 .getEntryLogManager();
@@ -589,8 +605,11 @@ public class LedgerStorageCheckpointTest {
 
         Assert.assertEquals("Number of JournalDirs", 1, conf.getJournalDirs().length);
         // we know there is only one ledgerDir
-        File ledgerDir = Bookie.getCurrentDirectories(conf.getLedgerDirs())[0];
-        BookieServer server = new BookieServer(conf);
+        File ledgerDir = BookieImpl.getCurrentDirectories(conf.getLedgerDirs())[0];
+        BookieServer server = new BookieServer(
+                conf, new TestBookieImpl(conf),
+                NullStatsLogger.INSTANCE, PooledByteBufAllocator.DEFAULT,
+                new MockUncleanShutdownDetection());
         server.start();
         ClientConfiguration clientConf = new ClientConfiguration();
         clientConf.setMetadataServiceUri(zkUtil.getMetadataServiceUri());
@@ -653,7 +672,7 @@ public class LedgerStorageCheckpointTest {
         // Journal file
         File[] journalDirs = conf.getJournalDirs();
         for (File journalDir : journalDirs) {
-            File journalDirectory = Bookie.getCurrentDirectory(journalDir);
+            File journalDirectory = BookieImpl.getCurrentDirectory(journalDir);
             List<Long> journalLogsId = Journal.listJournalIds(journalDirectory, null);
             for (long journalId : journalLogsId) {
                 File journalFile = new File(journalDirectory, Long.toHexString(journalId) + ".txn");
@@ -667,7 +686,10 @@ public class LedgerStorageCheckpointTest {
 
         // now we are restarting BookieServer
         conf.setLedgerStorageClass(InterleavedLedgerStorage.class.getName());
-        server = new BookieServer(conf);
+        server = new BookieServer(
+                conf, new TestBookieImpl(conf),
+                NullStatsLogger.INSTANCE, PooledByteBufAllocator.DEFAULT,
+                new MockUncleanShutdownDetection());
         server.start();
         BookKeeper newBKClient = new BookKeeper(clientConf);
         // since Bookie checkpointed successfully before shutdown/crash,
