@@ -30,7 +30,6 @@ import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertThat;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
-
 import com.google.common.net.InetAddresses;
 import java.io.File;
 import java.util.ArrayList;
@@ -46,8 +45,10 @@ import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
-
-import org.apache.bookkeeper.bookie.Bookie;
+import org.apache.bookkeeper.bookie.BookieImpl;
+import org.apache.bookkeeper.bookie.BookieResources;
+import org.apache.bookkeeper.bookie.CookieValidation;
+import org.apache.bookkeeper.bookie.LegacyCookieValidation;
 import org.apache.bookkeeper.client.BookKeeper.DigestType;
 import org.apache.bookkeeper.client.api.LedgerMetadata;
 import org.apache.bookkeeper.common.component.ComponentStarter;
@@ -57,6 +58,8 @@ import org.apache.bookkeeper.conf.ClientConfiguration;
 import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.conf.TestBKConfiguration;
 import org.apache.bookkeeper.discover.BookieServiceInfo;
+import org.apache.bookkeeper.discover.RegistrationManager;
+import org.apache.bookkeeper.meta.MetadataBookieDriver;
 import org.apache.bookkeeper.meta.UnderreplicatedLedger;
 import org.apache.bookkeeper.meta.ZkLedgerUnderreplicationManager;
 import org.apache.bookkeeper.meta.zk.ZKMetadataDriverBase;
@@ -66,6 +69,7 @@ import org.apache.bookkeeper.proto.BookieServer;
 import org.apache.bookkeeper.replication.ReplicationException.UnavailableException;
 import org.apache.bookkeeper.server.Main;
 import org.apache.bookkeeper.server.conf.BookieConfiguration;
+import org.apache.bookkeeper.stats.NullStatsLogger;
 import org.apache.bookkeeper.test.BookKeeperClusterTestCase;
 import org.apache.bookkeeper.util.AvailabilityOfEntriesOfLedger;
 import org.apache.bookkeeper.util.BookKeeperConstants;
@@ -124,10 +128,10 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
     }
 
     public void testTriggerAudit(boolean storeSystemTimeAsLedgerUnderreplicatedMarkTime) throws Exception {
-        ServerConfiguration thisServerConf = new ServerConfiguration(baseConf);
-        thisServerConf
-                .setStoreSystemTimeAsLedgerUnderreplicatedMarkTime(storeSystemTimeAsLedgerUnderreplicatedMarkTime);
-        restartBookies(thisServerConf);
+        restartBookies(c -> {
+                c.setStoreSystemTimeAsLedgerUnderreplicatedMarkTime(storeSystemTimeAsLedgerUnderreplicatedMarkTime);
+                return c;
+            });
         ClientConfiguration thisClientConf = new ClientConfiguration(baseClientConf);
         thisClientConf
                 .setStoreSystemTimeAsLedgerUnderreplicatedMarkTime(storeSystemTimeAsLedgerUnderreplicatedMarkTime);
@@ -152,7 +156,7 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
         ledgerHandle.addEntry(0, "data".getBytes());
         ledgerHandle.close();
 
-        BookieServer bookieToKill = bs.get(1);
+        BookieServer bookieToKill = serverByIndex(1);
         killBookie(1);
         /*
          * since lostBookieRecoveryDelay is set, when a bookie is died, it will
@@ -165,8 +169,8 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
         assertTrue("There are supposed to be underreplicatedledgers", underreplicatedLedgerItr.hasNext());
         UnderreplicatedLedger underreplicatedLedger = underreplicatedLedgerItr.next();
         assertEquals("Underreplicated ledgerId", ledgerId, underreplicatedLedger.getLedgerId());
-        assertTrue("Missingreplica of Underreplicated ledgerId should contain " + bookieToKill.getBookieId(),
-                underreplicatedLedger.getReplicaList().contains(bookieToKill.getBookieId().toString()));
+        assertTrue("Missingreplica of Underreplicated ledgerId should contain " + bookieToKill,
+                underreplicatedLedger.getReplicaList().contains(bookieToKill.getBookieId().getId()));
         if (storeSystemTimeAsLedgerUnderreplicatedMarkTime) {
             long ctimeOfURL = underreplicatedLedger.getCtime();
             assertTrue("ctime of underreplicated ledger should be greater than test starttime",
@@ -180,18 +184,19 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
 
     @Test
     public void testBookieInit() throws Exception {
-        int bookieindex = 0;
-        ServerConfiguration confOfExistingBookie = bsConfs.get(bookieindex);
-        Assert.assertFalse("initBookie shouldn't have succeeded, since bookie is still running with that configuration",
-                BookKeeperAdmin.initBookie(confOfExistingBookie));
-        killBookie(bookieindex);
-        Assert.assertFalse("initBookie shouldn't have succeeded, since previous bookie is not formatted yet",
-                BookKeeperAdmin.initBookie(confOfExistingBookie));
-
-        File[] journalDirs = confOfExistingBookie.getJournalDirs();
-        for (File journalDir : journalDirs) {
-            FileUtils.deleteDirectory(journalDir);
+        ServerConfiguration confOfExistingBookie = newServerConfiguration();
+        BookieId bookieId = BookieImpl.getBookieId(confOfExistingBookie);
+        try (MetadataBookieDriver driver = BookieResources.createMetadataDriver(
+                confOfExistingBookie, NullStatsLogger.INSTANCE);
+             RegistrationManager rm = driver.createRegistrationManager()) {
+            CookieValidation cookieValidation = new LegacyCookieValidation(confOfExistingBookie, rm);
+            cookieValidation.checkCookies(Main.storageDirectoriesFromConf(confOfExistingBookie));
+            rm.registerBookie(bookieId, false /* readOnly */, BookieServiceInfo.EMPTY);
+            Assert.assertFalse(
+                    "initBookie shouldn't have succeeded, since bookie is still running with that configuration",
+                    BookKeeperAdmin.initBookie(confOfExistingBookie));
         }
+
         Assert.assertFalse("initBookie shouldn't have succeeded, since previous bookie is not formatted yet completely",
                 BookKeeperAdmin.initBookie(confOfExistingBookie));
 
@@ -210,11 +215,10 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
         }
         Assert.assertFalse("initBookie shouldn't have succeeded, since cookie in ZK is not deleted yet",
                 BookKeeperAdmin.initBookie(confOfExistingBookie));
-        String bookieId = Bookie.getBookieId(confOfExistingBookie).toString();
         String bookieCookiePath =
             ZKMetadataDriverBase.resolveZkLedgersRootPath(confOfExistingBookie)
                 + "/" + BookKeeperConstants.COOKIE_NODE
-                + "/" + bookieId;
+                + "/" + bookieId.toString();
         zkc.delete(bookieCookiePath, -1);
 
         Assert.assertTrue("initBookie shouldn't succeeded",
@@ -439,10 +443,11 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
     @Test
     public void testGetListOfEntriesOfNonExistingLedger() throws Exception {
         long nonExistingLedgerId = 56789L;
+
         try (BookKeeperAdmin bkAdmin = new BookKeeperAdmin(zkUtil.getZooKeeperConnectString())) {
-            for (int i = 0; i < bs.size(); i++) {
+            for (int i = 0; i < bookieCount(); i++) {
                 CompletableFuture<AvailabilityOfEntriesOfLedger> futureResult = bkAdmin
-                        .asyncGetListOfEntriesOfLedger(bs.get(i).getBookieId(), nonExistingLedgerId);
+                    .asyncGetListOfEntriesOfLedger(addressByIndex(i), nonExistingLedgerId);
                 try {
                     futureResult.get();
                     fail("asyncGetListOfEntriesOfLedger is supposed to be failed with NoSuchLedgerExistsException");
@@ -469,9 +474,9 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
             lh.close();
         }
         try (BookKeeperAdmin bkAdmin = new BookKeeperAdmin(zkUtil.getZooKeeperConnectString())) {
-            for (int i = 0; i < bs.size(); i++) {
+            for (int i = 0; i < bookieCount(); i++) {
                 CompletableFuture<AvailabilityOfEntriesOfLedger> futureResult = bkAdmin
-                        .asyncGetListOfEntriesOfLedger(bs.get(i).getBookieId(), lId);
+                    .asyncGetListOfEntriesOfLedger(addressByIndex(i), lId);
                 AvailabilityOfEntriesOfLedger availabilityOfEntriesOfLedger = futureResult.get();
                 assertEquals("Number of entries", numOfEntries,
                         availabilityOfEntriesOfLedger.getTotalNumOfAvailableEntries());
@@ -485,6 +490,23 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
         bkc.close();
     }
 
+    @Test
+    public void testGetEntriesFromEmptyLedger() throws Exception {
+        ClientConfiguration conf = new ClientConfiguration();
+        conf.setMetadataServiceUri(zkUtil.getMetadataServiceUri());
+        BookKeeper bkc = new BookKeeper(conf);
+        LedgerHandle lh = bkc.createLedger(numOfBookies, numOfBookies, digestType, "testPasswd".getBytes(UTF_8));
+        lh.close();
+        long ledgerId = lh.getId();
+
+        try (BookKeeperAdmin bkAdmin = new BookKeeperAdmin(zkUtil.getZooKeeperConnectString())) {
+            Iterator<LedgerEntry> iter = bkAdmin.readEntries(ledgerId, 0, 0).iterator();
+            assertFalse(iter.hasNext());
+        }
+
+        bkc.close();
+    }
+
     @Test
     public void testGetListOfEntriesOfLedgerWithJustOneBookieInWriteQuorum() throws Exception {
         ClientConfiguration conf = new ClientConfiguration();
@@ -502,9 +524,9 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
         }
 
         try (BookKeeperAdmin bkAdmin = new BookKeeperAdmin(zkUtil.getZooKeeperConnectString())) {
-            for (int i = 0; i < bs.size(); i++) {
+            for (int i = 0; i < bookieCount(); i++) {
                 CompletableFuture<AvailabilityOfEntriesOfLedger> futureResult = bkAdmin
-                        .asyncGetListOfEntriesOfLedger(bs.get(i).getBookieId(), lId);
+                        .asyncGetListOfEntriesOfLedger(addressByIndex(i), lId);
                 AvailabilityOfEntriesOfLedger availabilityOfEntriesOfLedger = futureResult.get();
                 /*
                  * since num of bookies in the ensemble is 2 and
@@ -531,22 +553,22 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
 
         try (BookKeeperAdmin bkAdmin = new BookKeeperAdmin(zkUtil.getZooKeeperConnectString())) {
             Collection<BookieId> availableBookies = bkAdmin.getAvailableBookies();
-            Assert.assertEquals(availableBookies.size(), bs.size());
+            Assert.assertEquals(availableBookies.size(), bookieCount());
 
-            for (int i = 0; i < bs.size(); i++) {
-                availableBookies.contains(bs.get(i).getBookieId());
+            for (int i = 0; i < bookieCount(); i++) {
+                availableBookies.contains(addressByIndex(i));
             }
 
-            BookieServer killedBookie = bs.get(1);
+            BookieServer killedBookie = serverByIndex(1);
             killBookieAndWaitForZK(1);
 
             Collection<BookieId> remainingBookies = bkAdmin.getAvailableBookies();
-            Assert.assertFalse(remainingBookies.contains(killedBookie.getBookieId()));
+            Assert.assertFalse(remainingBookies.contains(killedBookie));
 
             Collection<BookieId> allBookies = bkAdmin.getAllBookies();
-            for (int i = 0; i < bs.size(); i++) {
-                remainingBookies.contains(bs.get(i).getBookieId());
-                allBookies.contains(bs.get(i).getBookieId());
+            for (int i = 0; i < bookieCount(); i++) {
+                remainingBookies.contains(addressByIndex(i));
+                allBookies.contains(addressByIndex(i));
             }
 
             Assert.assertEquals(remainingBookies.size(), allBookies.size() - 1);
@@ -577,7 +599,7 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
          * since no entry is added, callback is supposed to fail with
          * NoSuchLedgerExistsException.
          */
-        bkAdmin.asyncGetListOfEntriesOfLedger(bs.get(0).getBookieId(), lId)
+        bkAdmin.asyncGetListOfEntriesOfLedger(addressByIndex(0), lId)
                 .whenComplete((availabilityOfEntriesOfLedger, throwable) -> {
                     exceptionInCallback.set(throwable != null);
                     if (throwable != null) {
@@ -666,7 +688,7 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
     }
 
     private void testBookieServiceInfo(boolean readonly, boolean legacy) throws Exception {
-        File tmpDir = createTempDir("bookie", "test");
+        File tmpDir = tmpDirs.createNew("bookie", "test");
         final ServerConfiguration conf = TestBKConfiguration.newServerConfiguration()
                 .setJournalDirName(tmpDir.getPath())
                 .setLedgerDirNames(new String[]{tmpDir.getPath()})
@@ -681,9 +703,9 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
         }
 
         ServerConfiguration bkConf = newServerConfiguration().setForceReadOnlyBookie(readonly);
-        BookieServer bkServer = startBookie(bkConf);
+        BookieServer bkServer = startBookie(bkConf).getServer();
 
-        String bookieId = bkServer.getBookieId().toString();
+        BookieId bookieId = bkServer.getBookieId();
         String host = bkServer.getLocalAddress().getHostName();
         int port = bkServer.getLocalAddress().getPort();
 
@@ -691,7 +713,7 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
             String regPath = ZKMetadataDriverBase.resolveZkLedgersRootPath(bkConf) + "/" + AVAILABLE_NODE;
             regPath = readonly
                     ? regPath + READONLY + "/" + bookieId
-                    : regPath + "/" + bookieId;
+                    : regPath + "/" + bookieId.toString();
             // deleting the metadata, so that the bookie registration should
             // continue successfully with legacy BookieServiceInfo
             zkc.setData(regPath, new byte[]{}, -1);
@@ -702,7 +724,7 @@ public class BookKeeperAdminTest extends BookKeeperClusterTestCase {
 
             assertThat(bookieServiceInfo.getEndpoints().size(), is(1));
             BookieServiceInfo.Endpoint endpoint = bookieServiceInfo.getEndpoints().stream()
-                    .filter(e -> Objects.equals(e.getId(), bookieId))
+                    .filter(e -> Objects.equals(e.getId(), bookieId.getId()))
                     .findFirst()
                     .get();
             assertNotNull("Endpoint " + bookieId + " not found.", endpoint);
