@@ -38,6 +38,7 @@ import java.util.function.BiConsumer;
 
 import org.apache.bookkeeper.bookie.Bookie;
 import org.apache.bookkeeper.bookie.BookieException;
+import org.apache.bookkeeper.bookie.TestBookieImpl;
 import org.apache.bookkeeper.client.AsyncCallback.AddCallback;
 import org.apache.bookkeeper.client.AsyncCallback.CloseCallback;
 import org.apache.bookkeeper.client.AsyncCallback.CreateCallback;
@@ -75,7 +76,7 @@ public class BookKeeperCloseTest extends BookKeeperClusterTestCase {
     private void restartBookieSlow() throws Exception{
         ServerConfiguration conf = killBookie(0);
 
-        Bookie delayBookie = new Bookie(conf) {
+        Bookie delayBookie = new TestBookieImpl(conf) {
                 @Override
                 public void recoveryAddEntry(ByteBuf entry, WriteCallback cb,
                                              Object ctx, byte[] masterKey)
@@ -106,7 +107,7 @@ public class BookKeeperCloseTest extends BookKeeperClusterTestCase {
 
                 @Override
                 public ByteBuf readEntry(long ledgerId, long entryId)
-                        throws IOException, NoLedgerException {
+                        throws IOException, NoLedgerException, BookieException {
                     try {
                         Thread.sleep(5000);
                     } catch (InterruptedException ie) {
@@ -117,8 +118,7 @@ public class BookKeeperCloseTest extends BookKeeperClusterTestCase {
                     return super.readEntry(ledgerId, entryId);
                 }
             };
-        bsConfs.add(conf);
-        bs.add(startBookie(conf, delayBookie));
+        startAndAddBookie(conf, delayBookie);
     }
 
     /**
