@@ -22,7 +22,6 @@ package org.apache.bookkeeper.client;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.fail;
-
 import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.client.BKException.BKBookieHandleNotAvailableException;
 import org.apache.bookkeeper.client.api.BookKeeper;
@@ -30,10 +29,8 @@ import org.apache.bookkeeper.client.api.LedgerEntries;
 import org.apache.bookkeeper.client.api.ReadHandle;
 import org.apache.bookkeeper.client.api.WriteHandle;
 import org.apache.bookkeeper.conf.ClientConfiguration;
-import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.discover.ZKRegistrationClient;
 import org.apache.bookkeeper.test.BookKeeperClusterTestCase;
-import org.apache.bookkeeper.util.PortManager;
 import org.junit.Ignore;
 import org.junit.Test;
 
@@ -69,9 +66,7 @@ public class BookieNetworkAddressChangeTest extends BookKeeperClusterTestCase {
 
             // restart bookie, change port
             // on metadata we have a bookieId, not the network address
-            ServerConfiguration thisServerConf = new ServerConfiguration(baseConf);
-            thisServerConf.setBookiePort(PortManager.nextFreePort());
-            restartBookies(thisServerConf);
+            restartBookies(c -> c);
 
             try (ReadHandle h = bkc
                     .newOpenLedgerOp()
@@ -156,9 +151,7 @@ public class BookieNetworkAddressChangeTest extends BookKeeperClusterTestCase {
 
             // restart bookie, change port
             // on metadata we have a bookieId, not the network address
-            ServerConfiguration thisServerConf = new ServerConfiguration(baseConf);
-            thisServerConf.setBookiePort(PortManager.nextFreePort());
-            restartBookies(thisServerConf);
+            restartBookies(c -> c);
 
             try (ReadHandle h = bkc
                     .newOpenLedgerOp()
