@@ -23,10 +23,8 @@ package org.apache.bookkeeper.client;
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
-
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.Unpooled;
-
 import java.io.IOException;
 import java.util.Enumeration;
 import java.util.concurrent.CompletableFuture;
@@ -38,10 +36,9 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
-
-import org.apache.bookkeeper.bookie.Bookie;
 import org.apache.bookkeeper.bookie.BookieException;
 import org.apache.bookkeeper.bookie.InterleavedLedgerStorage;
+import org.apache.bookkeeper.bookie.TestBookieImpl;
 import org.apache.bookkeeper.client.BookKeeper.DigestType;
 import org.apache.bookkeeper.client.api.LedgerMetadata;
 import org.apache.bookkeeper.client.api.WriteFlag;
@@ -68,7 +65,6 @@ import org.apache.bookkeeper.versioning.Version;
 import org.apache.bookkeeper.versioning.Versioned;
 import org.apache.commons.lang3.mutable.MutableInt;
 import org.apache.zookeeper.AsyncCallback.VoidCallback;
-import org.apache.zookeeper.KeeperException;
 import org.junit.After;
 import org.junit.Test;
 import org.slf4j.Logger;
@@ -486,7 +482,7 @@ public class ParallelLedgerRecoveryTest extends BookKeeperClusterTestCase {
         assertEquals("recovery callback should be triggered only once", 0, numFailureCalls.get());
     }
 
-    static class DelayResponseBookie extends Bookie {
+    static class DelayResponseBookie extends TestBookieImpl {
 
         static final class WriteCallbackEntry {
 
@@ -521,7 +517,7 @@ public class ParallelLedgerRecoveryTest extends BookKeeperClusterTestCase {
                 new LinkedBlockingQueue<WriteCallbackEntry>();
 
         public DelayResponseBookie(ServerConfiguration conf)
-                throws IOException, KeeperException, InterruptedException, BookieException {
+                throws Exception {
             super(conf);
         }
 
@@ -542,7 +538,7 @@ public class ParallelLedgerRecoveryTest extends BookKeeperClusterTestCase {
         }
 
         @Override
-        public ByteBuf readEntry(long ledgerId, long entryId) throws IOException, NoLedgerException {
+        public ByteBuf readEntry(long ledgerId, long entryId) throws IOException, NoLedgerException, BookieException {
             LOG.info("ReadEntry {} - {}", ledgerId, entryId);
             if (delayReadResponse.get() && delayReadOnEntry.get() == entryId) {
                 CountDownLatch latch = delayReadLatch;
@@ -598,8 +594,7 @@ public class ParallelLedgerRecoveryTest extends BookKeeperClusterTestCase {
         ServerConfiguration conf = killBookie(address);
         conf.setLedgerStorageClass(InterleavedLedgerStorage.class.getName());
         DelayResponseBookie fakeBookie = new DelayResponseBookie(conf);
-        bs.add(startBookie(conf, fakeBookie));
-        bsConfs.add(conf);
+        startAndAddBookie(conf, fakeBookie);
 
         // 1) bk0 write two entries
         lh0.addEntry("entry-0".getBytes(UTF_8));
