@@ -28,6 +28,7 @@ import java.io.IOException;
 import java.net.URI;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.EnumSet;
 import java.util.Map;
 import java.util.NavigableMap;
 import java.util.Optional;
@@ -174,13 +175,17 @@ public abstract class LedgerManagerTestCase extends BookKeeperClusterTestCase {
             LedgerManager ledgerManager,
             LedgerDirsManager ledgerDirsManager,
             LedgerDirsManager indexDirsManager,
-            StateManager stateManager,
-            CheckpointSource checkpointSource,
-            Checkpointer checkpointer,
             StatsLogger statsLogger,
             ByteBufAllocator allocator) throws IOException {
         }
 
+        @Override
+        public void setStateManager(StateManager stateManager) {}
+        @Override
+        public void setCheckpointSource(CheckpointSource checkpointSource) {}
+        @Override
+        public void setCheckpointer(Checkpointer checkpointer) {}
+
         @Override
         public void start() {
         }
@@ -194,6 +199,11 @@ public abstract class LedgerManagerTestCase extends BookKeeperClusterTestCase {
             return false;
         }
 
+        @Override
+        public boolean entryExists(long ledgerId, long entryId) throws IOException {
+            return false;
+        }
+
         @Override
         public boolean setFenced(long ledgerId) throws IOException {
             return false;
@@ -294,5 +304,36 @@ public abstract class LedgerManagerTestCase extends BookKeeperClusterTestCase {
         public OfLong getListOfEntriesOfLedger(long ledgerId) {
             return null;
         }
+
+        @Override
+        public void setLimboState(long ledgerId) throws IOException {
+            throw new UnsupportedOperationException(
+                    "Limbo state only supported for DbLedgerStorage");
+        }
+
+        @Override
+        public boolean hasLimboState(long ledgerId) throws IOException {
+            throw new UnsupportedOperationException(
+                    "Limbo state only supported for DbLedgerStorage");
+        }
+
+        @Override
+        public void clearLimboState(long ledgerId) throws IOException {
+            throw new UnsupportedOperationException(
+                    "Limbo state only supported for DbLedgerStorage");
+        }
+
+        @Override
+        public EnumSet<StorageState> getStorageStateFlags() throws IOException {
+            return EnumSet.noneOf(StorageState.class);
+        }
+
+        @Override
+        public void setStorageStateFlag(StorageState flag) throws IOException {
+        }
+
+        @Override
+        public void clearStorageStateFlag(StorageState flag) throws IOException {
+        }
     }
 }
