@@ -20,6 +20,7 @@
  */
 package org.apache.bookkeeper.proto;
 
+import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 
@@ -28,6 +29,9 @@ import com.google.protobuf.ExtensionRegistry;
 import io.netty.buffer.ByteBuf;
 import io.netty.channel.Channel;
 import io.netty.channel.EventLoopGroup;
+import io.netty.channel.epoll.Epoll;
+import io.netty.channel.epoll.EpollChannelOption;
+import io.netty.channel.epoll.EpollEventLoopGroup;
 import io.netty.channel.nio.NioEventLoopGroup;
 
 import java.io.IOException;
@@ -38,6 +42,8 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import org.apache.bookkeeper.auth.AuthProviderFactoryFactory;
 import org.apache.bookkeeper.auth.ClientAuthProvider;
 import org.apache.bookkeeper.bookie.Bookie;
+import org.apache.bookkeeper.bookie.BookieException;
+import org.apache.bookkeeper.bookie.TestBookieImpl;
 import org.apache.bookkeeper.client.BKException;
 import org.apache.bookkeeper.common.util.OrderedExecutor;
 import org.apache.bookkeeper.conf.ClientConfiguration;
@@ -47,9 +53,12 @@ import org.apache.bookkeeper.net.BookieSocketAddress;
 import org.apache.bookkeeper.proto.BookkeeperInternalCallbacks.GenericCallback;
 import org.apache.bookkeeper.proto.BookkeeperInternalCallbacks.ReadEntryCallback;
 import org.apache.bookkeeper.proto.PerChannelBookieClient.ConnectionState;
+import org.apache.bookkeeper.stats.StatsLogger;
 import org.apache.bookkeeper.test.BookKeeperClusterTestCase;
 import org.apache.bookkeeper.util.SafeRunnable;
+import org.junit.Assume;
 import org.junit.Test;
+import org.mockito.Mockito;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -236,10 +245,10 @@ public class TestPerChannelBookieClient extends BookKeeperClusterTestCase {
     public void testRequestCompletesAfterDisconnectRace() throws Exception {
         ServerConfiguration conf = killBookie(0);
 
-        Bookie delayBookie = new Bookie(conf) {
+        Bookie delayBookie = new TestBookieImpl(conf) {
             @Override
             public ByteBuf readEntry(long ledgerId, long entryId)
-                    throws IOException, NoLedgerException {
+                    throws IOException, NoLedgerException, BookieException {
                 try {
                     Thread.sleep(3000);
                 } catch (InterruptedException ie) {
@@ -249,8 +258,7 @@ public class TestPerChannelBookieClient extends BookKeeperClusterTestCase {
                 return super.readEntry(ledgerId, entryId);
             }
         };
-        bsConfs.add(conf);
-        bs.add(startBookie(conf, delayBookie));
+        startAndAddBookie(conf, delayBookie);
 
         EventLoopGroup eventLoopGroup = new NioEventLoopGroup();
         final OrderedExecutor executor = getOrderedSafeExecutor();
@@ -295,4 +303,42 @@ public class TestPerChannelBookieClient extends BookKeeperClusterTestCase {
         eventLoopGroup.shutdownGracefully();
         executor.shutdown();
     }
+
+    /**
+     * Test that TCP user timeout is correctly set in EpollEventLoopGroup.
+     */
+    @Test
+    public void testEpollChannelTcpUserTimeout() throws Exception {
+        // Epoll is needed for this test to work.
+        Assume.assumeTrue(Epoll.isAvailable());
+
+        EventLoopGroup eventLoopGroup = new EpollEventLoopGroup();
+        OrderedExecutor executor = getOrderedSafeExecutor();
+        ClientConfiguration conf = new ClientConfiguration();
+        int tcpUserTimeout = 1236; // this value may be rounded on some Linux implementations
+        BookieId addr = getBookie(0);
+
+        // Pass to the PerChannelBookieClient object the client configuration with TCP user timeout.
+        PerChannelBookieClient channel = new PerChannelBookieClient(conf, executor, eventLoopGroup,
+                addr, Mockito.mock(StatsLogger.class), authProvider, extRegistry,
+                Mockito.mock(PerChannelBookieClientPool.class), BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
+
+        // Verify that the configured value has not been set in the channel if does not exist in config.
+        assertEquals(channel.connect().channel().config()
+                .getOption(EpollChannelOption.TCP_USER_TIMEOUT).intValue(), 0);
+        channel.close();
+
+        // Create a new channel with new TCP user timeout set.
+        conf.setTcpUserTimeoutMillis(tcpUserTimeout);
+        channel = new PerChannelBookieClient(conf, executor, eventLoopGroup,
+                addr, Mockito.mock(StatsLogger.class), authProvider, extRegistry,
+                Mockito.mock(PerChannelBookieClientPool.class), BookieSocketAddress.LEGACY_BOOKIEID_RESOLVER);
+
+        // Verify that the configured value has been set.
+        assertEquals(channel.connect().channel().config()
+                        .getOption(EpollChannelOption.TCP_USER_TIMEOUT).intValue(), tcpUserTimeout);
+        channel.close();
+        eventLoopGroup.shutdownGracefully();
+        executor.shutdown();
+    }
 }
