@@ -24,7 +24,6 @@ import static org.apache.bookkeeper.client.RackawareEnsemblePlacementPolicyImpl.
 import static org.apache.bookkeeper.replication.ReplicationStats.AUDITOR_SCOPE;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
-
 import java.net.URI;
 import java.net.UnknownHostException;
 import java.util.ArrayList;
@@ -33,8 +32,7 @@ import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
-
-import org.apache.bookkeeper.bookie.Bookie;
+import org.apache.bookkeeper.bookie.BookieImpl;
 import org.apache.bookkeeper.client.LedgerMetadataBuilder;
 import org.apache.bookkeeper.client.RackawareEnsemblePlacementPolicy;
 import org.apache.bookkeeper.client.ZoneawareEnsemblePlacementPolicy;
@@ -84,9 +82,8 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
     public void setUp() throws Exception {
         super.setUp();
         StaticDNSResolver.reset();
-        driver = MetadataDrivers.getBookieDriver(URI.create(bsConfs.get(0).getMetadataServiceUri()));
-        driver.initialize(bsConfs.get(0), () -> {
-        }, NullStatsLogger.INSTANCE);
+        driver = MetadataDrivers.getBookieDriver(URI.create(confByIndex(0).getMetadataServiceUri()));
+        driver.initialize(confByIndex(0), NullStatsLogger.INSTANCE);
     }
 
     @After
@@ -103,14 +100,14 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
         int numOfBookies = 5;
         List<BookieId> bookieAddresses = new ArrayList<>();
         BookieSocketAddress bookieAddress;
-        RegistrationManager regManager = driver.getRegistrationManager();
-
-        // all the numOfBookies (5) are going to be in different racks
-        for (int i = 0; i < numOfBookies; i++) {
-            bookieAddress = new BookieSocketAddress("98.98.98." + i, 2181);
-            StaticDNSResolver.addNodeToRack(bookieAddress.getHostName(), "/rack" + (i));
-            bookieAddresses.add(bookieAddress.toBookieId());
-            regManager.registerBookie(bookieAddress.toBookieId(), false, BookieServiceInfo.EMPTY);
+        try (RegistrationManager regManager = driver.createRegistrationManager()) {
+            // all the numOfBookies (5) are going to be in different racks
+            for (int i = 0; i < numOfBookies; i++) {
+                bookieAddress = new BookieSocketAddress("98.98.98." + i, 2181);
+                StaticDNSResolver.addNodeToRack(bookieAddress.getHostName(), "/rack" + (i));
+                bookieAddresses.add(bookieAddress.toBookieId());
+                regManager.registerBookie(bookieAddress.toBookieId(), false, BookieServiceInfo.EMPTY);
+            }
         }
 
         LedgerManagerFactory mFactory = driver.getLedgerManagerFactory();
@@ -183,7 +180,7 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
                 .build();
         lm.createLedgerMetadata(4L, initMeta).get();
 
-        ServerConfiguration servConf = new ServerConfiguration(bsConfs.get(0));
+        ServerConfiguration servConf = new ServerConfiguration(confByIndex(0));
         servConf.setMinNumRacksPerWriteQuorum(minNumRacksPerWriteQuorumConfValue);
         setServerConfigPropertiesForRackPlacement(servConf);
         MutableObject<Auditor> auditorRef = new MutableObject<Auditor>();
@@ -218,12 +215,12 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
         int numOfBookies = 5;
         int numOfLedgersNotAdheringToPlacementPolicy = 0;
         List<BookieId> bookieAddresses = new ArrayList<>();
-        RegistrationManager regManager = driver.getRegistrationManager();
-
-        for (int i = 0; i < numOfBookies; i++) {
-            BookieId bookieAddress = new BookieSocketAddress("98.98.98." + i, 2181).toBookieId();
-            bookieAddresses.add(bookieAddress);
-            regManager.registerBookie(bookieAddress, false, BookieServiceInfo.EMPTY);
+        try (RegistrationManager regManager = driver.createRegistrationManager()) {
+            for (int i = 0; i < numOfBookies; i++) {
+                BookieId bookieAddress = new BookieSocketAddress("98.98.98." + i, 2181).toBookieId();
+                bookieAddresses.add(bookieAddress);
+                regManager.registerBookie(bookieAddress, false, BookieServiceInfo.EMPTY);
+            }
         }
 
         // only three racks
@@ -274,7 +271,7 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
                 .build();
         lm.createLedgerMetadata(2L, initMeta).get();
 
-        ServerConfiguration servConf = new ServerConfiguration(bsConfs.get(0));
+        ServerConfiguration servConf = new ServerConfiguration(confByIndex(0));
         servConf.setMinNumRacksPerWriteQuorum(minNumRacksPerWriteQuorumConfValue);
         setServerConfigPropertiesForRackPlacement(servConf);
         MutableObject<Auditor> auditorRef = new MutableObject<Auditor>();
@@ -315,12 +312,12 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
         int underreplicatedLedgerRecoveryGracePeriod = timeElapsed ? 1 : 1000;
         int numOfURLedgersElapsedRecoveryGracePeriod = 0;
         List<BookieId> bookieAddresses = new ArrayList<BookieId>();
-        RegistrationManager regManager = driver.getRegistrationManager();
-
-        for (int i = 0; i < numOfBookies; i++) {
-            BookieId bookieAddress = new BookieSocketAddress("98.98.98." + i, 2181).toBookieId();
-            bookieAddresses.add(bookieAddress);
-            regManager.registerBookie(bookieAddress, false, BookieServiceInfo.EMPTY);
+        try (RegistrationManager regManager = driver.createRegistrationManager()) {
+            for (int i = 0; i < numOfBookies; i++) {
+                BookieId bookieAddress = new BookieSocketAddress("98.98.98." + i, 2181).toBookieId();
+                bookieAddresses.add(bookieAddress);
+                regManager.registerBookie(bookieAddress, false, BookieServiceInfo.EMPTY);
+            }
         }
 
         LedgerManagerFactory mFactory = driver.getLedgerManagerFactory();
@@ -410,7 +407,7 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
             Thread.sleep(5000);
         }
 
-        ServerConfiguration servConf = new ServerConfiguration(bsConfs.get(0));
+        ServerConfiguration servConf = new ServerConfiguration(confByIndex(0));
         servConf.setUnderreplicatedLedgerRecoveryGracePeriod(underreplicatedLedgerRecoveryGracePeriod);
         setServerConfigPropertiesForRackPlacement(servConf);
         MutableObject<Auditor> auditorRef = new MutableObject<Auditor>();
@@ -434,12 +431,12 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
         int numOfBookies = 7;
         int numOfLedgersNotAdheringToPlacementPolicy = 0;
         List<BookieId> bookieAddresses = new ArrayList<>();
-        RegistrationManager regManager = driver.getRegistrationManager();
-
-        for (int i = 0; i < numOfBookies; i++) {
-            BookieId bookieAddress = new BookieSocketAddress("98.98.98." + i, 2181).toBookieId();
-            bookieAddresses.add(bookieAddress);
-            regManager.registerBookie(bookieAddress, false, BookieServiceInfo.EMPTY);
+        try (RegistrationManager regManager = driver.createRegistrationManager()) {
+            for (int i = 0; i < numOfBookies; i++) {
+                BookieId bookieAddress = new BookieSocketAddress("98.98.98." + i, 2181).toBookieId();
+                bookieAddresses.add(bookieAddress);
+                regManager.registerBookie(bookieAddress, false, BookieServiceInfo.EMPTY);
+            }
         }
 
         // only three racks
@@ -511,7 +508,7 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
         lm.createLedgerMetadata(2L, initMeta).get();
         numOfLedgersNotAdheringToPlacementPolicy++;
 
-        ServerConfiguration servConf = new ServerConfiguration(bsConfs.get(0));
+        ServerConfiguration servConf = new ServerConfiguration(confByIndex(0));
         servConf.setMinNumRacksPerWriteQuorum(minNumRacksPerWriteQuorumConfValue);
         setServerConfigPropertiesForRackPlacement(servConf);
         MutableObject<Auditor> auditorRef = new MutableObject<Auditor>();
@@ -539,25 +536,25 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
         int numOfLedgersNotAdheringToPlacementPolicy = 0;
         int numOfLedgersSoftlyAdheringToPlacementPolicy = 0;
         List<BookieId> bookieAddresses = new ArrayList<BookieId>();
-        RegistrationManager regManager = driver.getRegistrationManager();
-
-        /*
-         * 6 bookies - 3 zones and 2 uds
-         */
-        for (int i = 0; i < numOfBookies; i++) {
-            BookieSocketAddress bookieAddress = new BookieSocketAddress("98.98.98." + i, 2181);
-            bookieAddresses.add(bookieAddress.toBookieId());
-            regManager.registerBookie(bookieAddress.toBookieId(), false, BookieServiceInfo.EMPTY);
-            String zone = "/zone" + (i % 3);
-            String upgradeDomain = "/ud" + (i % 2);
-            String networkLocation = zone + upgradeDomain;
-            StaticDNSResolver.addNodeToRack(bookieAddress.getHostName(), networkLocation);
+        try (RegistrationManager regManager = driver.createRegistrationManager()) {
+            /*
+             * 6 bookies - 3 zones and 2 uds
+             */
+            for (int i = 0; i < numOfBookies; i++) {
+                BookieSocketAddress bookieAddress = new BookieSocketAddress("98.98.98." + i, 2181);
+                bookieAddresses.add(bookieAddress.toBookieId());
+                regManager.registerBookie(bookieAddress.toBookieId(), false, BookieServiceInfo.EMPTY);
+                String zone = "/zone" + (i % 3);
+                String upgradeDomain = "/ud" + (i % 2);
+                String networkLocation = zone + upgradeDomain;
+                StaticDNSResolver.addNodeToRack(bookieAddress.getHostName(), networkLocation);
+            }
         }
 
         LedgerManagerFactory mFactory = driver.getLedgerManagerFactory();
         LedgerManager lm = mFactory.newLedgerManager();
 
-        ServerConfiguration servConf = new ServerConfiguration(bsConfs.get(0));
+        ServerConfiguration servConf = new ServerConfiguration(confByIndex(0));
         servConf.setDesiredNumZonesPerWriteQuorum(3);
         servConf.setMinNumZonesPerWriteQuorum(2);
         setServerConfigPropertiesForZonePlacement(servConf);
@@ -687,7 +684,7 @@ public class AuditorPlacementPolicyCheckTest extends BookKeeperClusterTestCase {
         TestOpStatsLogger placementPolicyCheckStatsLogger = (TestOpStatsLogger) statsLogger
                 .getOpStatsLogger(ReplicationStats.PLACEMENT_POLICY_CHECK_TIME);
 
-        final TestAuditor auditor = new TestAuditor(Bookie.getBookieId(servConf).toString(), servConf,
+        final TestAuditor auditor = new TestAuditor(BookieImpl.getBookieId(servConf).toString(), servConf,
                 statsLogger);
         auditorRef.setValue(auditor);
         CountDownLatch latch = auditor.getLatch();
