@@ -46,11 +46,22 @@ public class PrometheusStatsLogger implements StatsLogger {
         return provider.opStats.computeIfAbsent(scopeContext(name), x -> new DataSketchesOpStatsLogger(labels));
     }
 
+    @Override
+    public OpStatsLogger getThreadScopedOpStatsLogger(String name) {
+        return provider.threadScopedOpStats.computeIfAbsent(scopeContext(name),
+                x -> new ThreadScopedDataSketchesStatsLogger(provider, x, labels));
+    }
+
     @Override
     public Counter getCounter(String name) {
         return provider.counters.computeIfAbsent(scopeContext(name), x -> new LongAdderCounter(labels));
     }
 
+    public Counter getThreadScopedCounter(String name) {
+        return provider.threadScopedCounters.computeIfAbsent(scopeContext(name),
+                x -> new ThreadScopedLongAdderCounter(provider, x, labels));
+    }
+
     @Override
     public <T extends Number> void registerGauge(String name, Gauge<T> gauge) {
         provider.gauges.computeIfAbsent(scopeContext(name), x -> new SimpleGauge<T>(gauge, labels));
