@@ -179,6 +179,9 @@ public class PrometheusTextFormat {
         Enumeration<MetricFamilySamples> metricFamilySamples = registry.metricFamilySamples();
         while (metricFamilySamples.hasMoreElements()) {
             MetricFamilySamples metricFamily = metricFamilySamples.nextElement();
+            // Write type of metric
+            w.append("# TYPE ").append(metricFamily.name).append(getTypeNameSuffix(metricFamily.type)).append(' ')
+                    .append(getTypeStr(metricFamily.type)).write('\n');
 
             for (int i = 0; i < metricFamily.samples.size(); i++) {
                 Sample sample = metricFamily.samples.get(i);
@@ -201,6 +204,25 @@ public class PrometheusTextFormat {
         }
     }
 
+    static String getTypeNameSuffix(Collector.Type type) {
+        return "";
+    }
+
+    static String getTypeStr(Collector.Type type) {
+        switch (type) {
+            case COUNTER:
+                return "counter";
+            case GAUGE:
+                return "gauge";
+            case SUMMARY:
+                return "summary";
+            case HISTOGRAM:
+                return "histogram";
+            default:
+                return "unknown";
+        }
+    }
+
     void writeType(Writer w, String name, String type) throws IOException {
         if (metricNameSet.contains(name)) {
             return;
