@@ -53,7 +53,7 @@ class EtcdBookieRegister implements AutoCloseable, Runnable, Supplier<Long> {
     private final Lease leaseClient;
     private final long ttlSeconds;
     private final ScheduledExecutorService executor;
-    private final RegistrationListener regListener;
+    private RegistrationListener regListener;
     private volatile CompletableFuture<Long> leaseFuture = new CompletableFuture<>();
     private volatile CompletableFuture<Void> keepAliveFuture = new CompletableFuture<>();
 
@@ -65,9 +65,7 @@ class EtcdBookieRegister implements AutoCloseable, Runnable, Supplier<Long> {
     private Future<?> runFuture = null;
 
     EtcdBookieRegister(Lease leaseClient,
-                       long ttlSeconds,
-                       RegistrationListener regListener) {
-        this.regListener = regListener;
+                       long ttlSeconds) {
         this.leaseClient = leaseClient;
         this.ttlSeconds = ttlSeconds;
         this.executor = Executors.newSingleThreadScheduledExecutor(
@@ -76,6 +74,11 @@ class EtcdBookieRegister implements AutoCloseable, Runnable, Supplier<Long> {
                 .build());
     }
 
+    public EtcdBookieRegister addRegistrationListener(RegistrationListener regListener) {
+        this.regListener = regListener;
+        return this;
+    }
+
     long getTtlSeconds() {
         return ttlSeconds;
     }
@@ -116,7 +119,7 @@ class EtcdBookieRegister implements AutoCloseable, Runnable, Supplier<Long> {
 
                 @Override
                 public void onCompleted() {
-                    log.info("{} lease completed! leaseId {}", leaseId);
+                    log.info("lease completed! leaseId {}", leaseId);
                     keepAliveFuture.cancel(true);
                 }
             });
@@ -133,7 +136,7 @@ class EtcdBookieRegister implements AutoCloseable, Runnable, Supplier<Long> {
                 newLeaseIfNeeded();
                 nextWaitTimeMs = 100L;
             } catch (MetadataStoreException e) {
-                log.error("Failed to grant a new lease for leaseId", leaseId, e);
+                log.error("Failed to grant a new lease for leaseId {}", leaseId, e);
                 try {
                     TimeUnit.MILLISECONDS.sleep(nextWaitTimeMs);
                     nextWaitTimeMs *= 2;
