@@ -114,9 +114,8 @@ class EtcdRegistrationManager implements RegistrationManager {
             scope,
             new EtcdBookieRegister(
                 client.getLeaseClient(),
-                ttlSeconds,
-                listener
-            ).start(),
+                ttlSeconds
+            ).addRegistrationListener(listener).start(),
             true);
     }
 
@@ -523,6 +522,11 @@ class EtcdRegistrationManager implements RegistrationManager {
         return nukeExistingCluster(kvClient, scope);
     }
 
+    @Override
+    public void addRegistrationListener(RegistrationListener listener) {
+        bkRegister.addRegistrationListener(listener);
+    }
+
     static boolean nukeExistingCluster(KV kvClient, String scope) throws Exception {
         ByteSequence rootScopeKey = ByteSequence.from(scope, UTF_8);
         GetResponse resp = msResult(kvClient.get(rootScopeKey));
