@@ -24,8 +24,10 @@ import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
-import com.coreos.jetcd.Client;
 import com.google.common.collect.Sets;
+
+import io.etcd.jetcd.Client;
+
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
