@@ -31,6 +31,8 @@ import org.junit.After;
 import org.junit.AfterClass;
 import org.junit.Before;
 import org.junit.BeforeClass;
+import org.junit.Rule;
+import org.junit.rules.Timeout;
 
 /**
  * A test base that setup etcd cluster for testing.
@@ -38,6 +40,9 @@ import org.junit.BeforeClass;
 @Slf4j
 public abstract class EtcdTestBase {
 
+    @Rule
+    public Timeout globalTimeout = Timeout.seconds(120);
+
     protected static EtcdContainer etcdContainer;
 
     @BeforeClass
