@@ -91,7 +91,7 @@ public class DigestTypeBenchmark {
         public BufferType bufferType;
         @Param
         public Digest digest;
-        @Param({"1024", "4086", "8192", "16384", "65536"})
+        @Param({"64", "1024", "4086", "8192", "16384", "65536"})
         public int entrySize;
 
         private DigestManager crc32;
@@ -168,7 +168,7 @@ public class DigestTypeBenchmark {
     @BenchmarkMode(Mode.Throughput)
     @OutputTimeUnit(TimeUnit.MILLISECONDS)
     @Warmup(iterations = 2, time = 3, timeUnit = TimeUnit.SECONDS)
-    @Measurement(iterations = 5, time = 12, timeUnit = TimeUnit.SECONDS)
+    @Measurement(iterations = 2, time = 10, timeUnit = TimeUnit.SECONDS)
     @Threads(2)
     @Fork(value = 1, warmups = 1)
     public void digestManager(MyState state) {
